@@ -62,7 +62,7 @@ var PlacesOrganizer = {
     this._places.selectItems([itemId]);
     // Forcefully expand all-bookmarks
     if (aQueryName == "AllBookmarks")
-      asContainer(this._places.selectedNode).containerOpen = true;
+      PlacesUtils.asContainer(this._places.selectedNode).containerOpen = true;
   },
 
   init: function PO_init() {
@@ -88,11 +88,6 @@ var PlacesOrganizer = {
     // Set up the search UI.
     PlacesSearchBox.init();
 
-#ifdef PLACES_QUERY_BUILDER
-    // Set up the advanced query builder UI
-    PlacesQueryBuilder.init();
-#endif
-
     window.addEventListener("AppCommand", this, true);
 #ifdef XP_MACOSX
     // 1. Map Edit->Find command to OrganizerCommand_find:all.  Need to map
@@ -221,7 +216,7 @@ var PlacesOrganizer = {
       return;
 
     var node = this._places.selectedNode;
-    var queries = asQuery(node).getQueries({});
+    var queries = PlacesUtils.asQuery(node).getQueries();
 
     // Items are only excluded on the left pane.
     var options = node.queryOptions.clone();
@@ -336,7 +331,7 @@ var PlacesOrganizer = {
   onTreeFocus: function PO_onTreeFocus(aEvent) {
     var currentView = aEvent.currentTarget;
     var selectedNodes = currentView.selectedNode ? [currentView.selectedNode] :
-                        this._content.getSelectionNodes();
+                        this._content.selectedNodes;
     this._fillDetailsPane(selectedNodes);
   },
 
@@ -348,7 +343,8 @@ var PlacesOrganizer = {
   },
 
   openSelectedNode: function PO_openSelectedNode(aEvent) {
-    PlacesUIUtils.openNodeWithEvent(this._content.selectedNode, aEvent);
+    PlacesUIUtils.openNodeWithEvent(this._content.selectedNode, aEvent,
+                                    this._content.treeBoxObject.view);
   },
 
   /**
@@ -356,7 +352,7 @@ var PlacesOrganizer = {
    * main places pane.
    */
   getCurrentOptions: function PO_getCurrentOptions() {
-    return asQuery(this._content.getResult().root).queryOptions;
+    return PlacesUtils.asQuery(this._content.result.root).queryOptions;
   },
 
   /**
@@ -364,7 +360,7 @@ var PlacesOrganizer = {
    * main places pane.
    */
   getCurrentQueries: function PO_getCurrentQueries() {
-    return asQuery(this._content.getResult().root).getQueries({});
+    return PlacesUtils.asQuery(this._content.result.root).getQueries();
   },
 
   /**
@@ -396,8 +392,7 @@ var PlacesOrganizer = {
       if (fp.file) {
         var importer = Cc["@mozilla.org/browser/places/import-export-service;1"].
                        getService(Ci.nsIPlacesImportExportService);
-        var file = fp.file.QueryInterface(Ci.nsILocalFile);
-        importer.importHTMLFromFile(file, false);
+        importer.importHTMLFromFile(fp.file, false);
       }
     }
   },
@@ -423,54 +418,36 @@ var PlacesOrganizer = {
    * Populates the restore menu with the dates of the backups available.
    */
   populateRestoreMenu: function PO_populateRestoreMenu() {
-    var restorePopup = document.getElementById("fileRestorePopup");
+    let restorePopup = document.getElementById("fileRestorePopup");
 
-    var dateSvc = Cc["@mozilla.org/intl/scriptabledateformat;1"].
+    let dateSvc = Cc["@mozilla.org/intl/scriptabledateformat;1"].
                   getService(Ci.nsIScriptableDateFormat);
 
-    // remove existing menu items
-    // last item is the restoreFromFile item
+    // Remove existing menu items.  Last item is the restoreFromFile item.
     while (restorePopup.childNodes.length > 1)
       restorePopup.removeChild(restorePopup.firstChild);
 
-    // get list of files
-    var localizedFilename = PlacesUtils.getString("bookmarksArchiveFilename");
-    var localizedFilenamePrefix = localizedFilename.substr(0, localizedFilename.indexOf("-"));
-    var fileList = [];
-    var files = this.bookmarksBackupDir.directoryEntries;
-    while (files.hasMoreElements()) {
-      var f = files.getNext().QueryInterface(Ci.nsIFile);
-      var rx = new RegExp("^(bookmarks|" + localizedFilenamePrefix +
-                          ")-([0-9]{4}-[0-9]{2}-[0-9]{2})\.json$");
-      if (!f.isHidden() && f.leafName.match(rx)) {
-        var date = f.leafName.match(rx)[2].replace(/-/g, "/");
-        var dateObj = new Date(date);
-        fileList.push({date: dateObj, filename: f.leafName});
-      }
-    }
-
-    fileList.sort(function PO_fileList_compare(a, b) {
-      return b.date - a.date;
-    });
-
-    if (fileList.length == 0)
+    let backupFiles = PlacesUtils.backups.entries;
+    if (backupFiles.length == 0)
       return;
 
-    // populate menu
-    for (var i = 0; i < fileList.length; i++) {
-      var m = restorePopup.insertBefore
-        (document.createElement("menuitem"),
-         document.getElementById("restoreFromFile"));
+    // Populate menu with backups.
+    for (let i = 0; i < backupFiles.length; i++) {
+      let backupDate = PlacesUtils.backups.getDateForFile(backupFiles[i]);
+      let m = restorePopup.insertBefore(document.createElement("menuitem"),
+                                        document.getElementById("restoreFromFile"));
       m.setAttribute("label",
                      dateSvc.FormatDate("",
                                         Ci.nsIScriptableDateFormat.dateFormatLong,
-                                        fileList[i].date.getFullYear(),
-                                        fileList[i].date.getMonth() + 1,
-                                        fileList[i].date.getDate()));
-      m.setAttribute("value", fileList[i].filename);
+                                        backupDate.getFullYear(),
+                                        backupDate.getMonth() + 1,
+                                        backupDate.getDate()));
+      m.setAttribute("value", backupFiles[i].leafName);
       m.setAttribute("oncommand",
                      "PlacesOrganizer.onRestoreMenuItemClick(this);");
     }
+
+    // Add the restoreFromFile item.
     restorePopup.insertBefore(document.createElement("menuseparator"),
                               document.getElementById("restoreFromFile"));
   },
@@ -479,14 +456,14 @@ var PlacesOrganizer = {
    * Called when a menuitem is selected from the restore menu.
    */
   onRestoreMenuItemClick: function PO_onRestoreMenuItemClick(aMenuItem) {
-    var dirSvc = Cc["@mozilla.org/file/directory_service;1"].
-                 getService(Ci.nsIProperties);
-    var bookmarksFile = dirSvc.get("ProfD", Ci.nsIFile);
-    bookmarksFile.append("bookmarkbackups");
-    bookmarksFile.append(aMenuItem.getAttribute("value"));
-    if (!bookmarksFile.exists())
-      return;
-    this.restoreBookmarksFromFile(bookmarksFile);
+    let backupName = aMenuItem.getAttribute("value");
+    let backupFiles = PlacesUtils.backups.entries;
+    for (let i = 0; i < backupFiles.length; i++) {
+      if (backupFiles[i].leafName == backupName) {
+        this.restoreBookmarksFromFile(backupFiles[i]);
+        break;
+      }
+    }
   },
 
   /**
@@ -562,30 +539,10 @@ var PlacesOrganizer = {
     var backupsDir = dirSvc.get("Desk", Ci.nsILocalFile);
     fp.displayDirectory = backupsDir;
 
-    fp.defaultString = PlacesUtils.getBackupFilename();
+    fp.defaultString = PlacesUtils.backups.getFilenameForDate();
 
-    if (fp.show() != Ci.nsIFilePicker.returnCancel) {
-      PlacesUtils.backupBookmarksToFile(fp.file);
-
-      // copy new backup to /backups dir (bug 424389)
-      var latestBackup = PlacesUtils.getMostRecentBackup();
-      if (!latestBackup || latestBackup != fp.file) {
-        latestBackup.remove(false);
-        var name = PlacesUtils.getBackupFilename();
-        fp.file.copyTo(this.bookmarksBackupDir, name);
-      }
-    }
-  },
-
-  get bookmarksBackupDir() {
-    delete this.bookmarksBackupDir;
-    var dirSvc = Cc["@mozilla.org/file/directory_service;1"].
-                 getService(Ci.nsIProperties);
-    var bookmarksBackupDir = dirSvc.get("ProfD", Ci.nsIFile);
-    bookmarksBackupDir.append("bookmarkbackups");
-    if (!bookmarksBackupDir.exists())
-      bookmarksBackupDir.create(Ci.nsIFile.DIRECTORY_TYPE, 0700);
-    return this.bookmarksBackupDir = bookmarksBackupDir;
+    if (fp.show() != Ci.nsIFilePicker.returnCancel)
+      PlacesUtils.backups.saveBookmarksToJSONFile(fp.file);
   },
 
   _paneDisabled: false,
@@ -652,7 +609,7 @@ var PlacesOrganizer = {
 
   onContentTreeSelect: function PO_onContentTreeSelect() {
     if (this._content.treeBoxObject.focused)
-      this._fillDetailsPane(this._content.getSelectionNodes());
+      this._fillDetailsPane(this._content.selectedNodes);
   },
 
   _fillDetailsPane: function PO__fillDetailsPane(aNodeList) {
@@ -826,12 +783,7 @@ var PlacesOrganizer = {
     // Get the place: uri for the query.
     // If the advanced query builder is showing, use that.
     var options = this.getCurrentOptions();
-
-#ifdef PLACES_QUERY_BUILDER
-    var queries = PlacesQueryBuilder.queries;
-#else
     var queries = this.getCurrentQueries();
-#endif
 
     var placeSpec = PlacesUtils.history.queriesToQueryString(queries,
                                                              queries.length,
@@ -977,12 +929,12 @@ var PlacesSearchBox = {
    */
   updateCollectionTitle: function PSB_updateCollectionTitle(title) {
     if (title)
-      this.searchFilter.emptyText =
+      this.searchFilter.placeholder =
         PlacesUIUtils.getFormattedString("searchCurrentDefault", [title]);
     else
-      this.searchFilter.emptyText = this.filterCollection == "history" ?
-                                    PlacesUIUtils.getString("searchHistory") :
-                                    PlacesUIUtils.getString("searchBookmarks");
+      this.searchFilter.placeholder = this.filterCollection == "history" ?
+                                      PlacesUIUtils.getString("searchHistory") :
+                                      PlacesUIUtils.getString("searchBookmarks");
   },
 
   /**
@@ -1035,12 +987,6 @@ var PlacesSearchBox = {
     // Hide the advanced search controls when the user hasn't searched
     var searchModifiers = document.getElementById("searchModifiers");
     searchModifiers.hidden = false;
-
-#ifdef PLACES_QUERY_BUILDER
-    // if new search, open builder with pre-populated text row
-    if (PlacesQueryBuilder.numRows == 0)
-      document.getElementById("OrganizerCommand_search:moreCriteria").doCommand();
-#endif
   },
 
   hideSearchUI: function PSB_hideSearchUI() {
@@ -1057,470 +1003,6 @@ var PlacesQueryBuilder = {
   queries: [],
   queryOptions: null,
 
-#ifdef PLACES_QUERY_BUILDER
-  numRows: 0,
-
-  /**
-   * The maximum number of terms that can be added.
-   */
-  _maxRows: null,
-
-  _keywordSearch: {
-    advancedSearch_N_Subject: "advancedSearch_N_SubjectKeyword",
-    advancedSearch_N_LocationMenulist: false,
-    advancedSearch_N_TimeMenulist: false,
-    advancedSearch_N_Textbox: "",
-    advancedSearch_N_TimePicker: false,
-    advancedSearch_N_TimeMenulist2: false
-  },
-  _locationSearch: {
-    advancedSearch_N_Subject: "advancedSearch_N_SubjectLocation",
-    advancedSearch_N_LocationMenulist: "advancedSearch_N_LocationMenuSelected",
-    advancedSearch_N_TimeMenulist: false,
-    advancedSearch_N_Textbox: "",
-    advancedSearch_N_TimePicker: false,
-    advancedSearch_N_TimeMenulist2: false
-  },
-  _timeSearch: {
-    advancedSearch_N_Subject: "advancedSearch_N_SubjectVisited",
-    advancedSearch_N_LocationMenulist: false,
-    advancedSearch_N_TimeMenulist: true,
-    advancedSearch_N_Textbox: false,
-    advancedSearch_N_TimePicker: "date",
-    advancedSearch_N_TimeMenulist2: false
-  },
-  _timeInLastSearch: {
-    advancedSearch_N_Subject: "advancedSearch_N_SubjectVisited",
-    advancedSearch_N_LocationMenulist: false,
-    advancedSearch_N_TimeMenulist: true,
-    advancedSearch_N_Textbox: "7",
-    advancedSearch_N_TimePicker: false,
-    advancedSearch_N_TimeMenulist2: true
-  },
-  _nextSearch: null,
-  _queryBuilders: null,
-
-
-  init: function PQB_init() {
-    // Initialize advanced search
-    this._nextSearch = {
-      "keyword": this._timeSearch,
-      "visited": this._locationSearch,
-      "location": null
-    };
-
-    this._queryBuilders = {
-      "keyword": this.setKeywordQuery,
-      "visited": this.setVisitedQuery,
-      "location": this.setLocationQuery
-    };
-
-    this._maxRows = this._queryBuilders.length;
-
-    this._dateService = Cc["@mozilla.org/intl/scriptabledateformat;1"].
-                        getService(Ci.nsIScriptableDateFormat);
-  },
-
-  /**
-   * Hides the query builder, and the match rule UI if visible.
-   */
-  hide: function PQB_hide() {
-    var advancedSearch = document.getElementById("advancedSearch");
-    // Need to collapse the advanced search box.
-    advancedSearch.collapsed = true;
-  },
-
-  /**
-   * Shows the query builder
-   */
-  show: function PQB_show() {
-    var advancedSearch = document.getElementById("advancedSearch");
-    advancedSearch.collapsed = false;
-  },
-
-  toggleVisibility: function ABP_toggleVisibility() {
-    var expander = document.getElementById("organizerScopeBarExpander");
-    var advancedSearch = document.getElementById("advancedSearch");
-    if (advancedSearch.collapsed) {
-      advancedSearch.collapsed = false;
-      expander.className = "expander-down";
-      expander.setAttribute("tooltiptext",
-                            expander.getAttribute("tooltiptextdown"));
-    }
-    else {
-      advancedSearch.collapsed = true;
-      expander.className = "expander-up"
-      expander.setAttribute("tooltiptext",
-                            expander.getAttribute("tooltiptextup"));
-    }
-  },
-
-  /**
-   * Includes the rowId in the id attribute of an element in a row newly
-   * created from the template row.
-   * @param   element
-   *          The element whose id attribute needs to be updated.
-   * @param   rowId
-   *          The index of the new row.
-   */
-  _setRowId: function PQB__setRowId(element, rowId) {
-    if (element.id)
-      element.id = element.id.replace("advancedSearch0", "advancedSearch" + rowId);
-    if (element.hasAttribute("rowid"))
-      element.setAttribute("rowid", rowId);
-    for (var i = 0; i < element.childNodes.length; ++i) {
-      this._setRowId(element.childNodes[i], rowId);
-    }
-  },
-
-  _updateUIForRowChange: function PQB__updateUIForRowChange() {
-    // Update the "can add more criteria" command to make sure various +
-    // buttons are disabled.
-    var command = document.getElementById("OrganizerCommand_search:moreCriteria");
-    if (this.numRows >= this._maxRows)
-      command.setAttribute("disabled", "true");
-    else
-      command.removeAttribute("disabled");
-  },
-
-  /**
-   * Adds a row to the view, prefilled with the next query subject. If the
-   * query builder is not visible, it will be shown.
-   */
-  addRow: function PQB_addRow() {
-    // Limits the number of rows that can be added based on the maximum number
-    // of search query subjects.
-    if (this.numRows >= this._maxRows)
-      return;
-
-    // Clone the template row and unset the hidden attribute.
-    var gridRows = document.getElementById("advancedSearchRows");
-    var newRow = gridRows.firstChild.cloneNode(true);
-    newRow.hidden = false;
-
-    // Determine what the search type is based on the last visible row. If this
-    // is the first row, the type is "keyword search". Otherwise, it's the next
-    // in the sequence after the one defined by the previous visible row's
-    // Subject selector, as defined in _nextSearch.
-    var searchType = this._keywordSearch;
-    var lastMenu = document.getElementById("advancedSearch" +
-                                           this.numRows +
-                                           "Subject");
-    if (this.numRows > 0 && lastMenu && lastMenu.selectedItem)
-      searchType = this._nextSearch[lastMenu.selectedItem.value];
-
-    // There is no "next" search type. We are here in error.
-    if (!searchType)
-      return;
-    // We don't insert into the document until _after_ the searchType is
-    // determined, since this will interfere with the computation.
-    gridRows.appendChild(newRow);
-    this._setRowId(newRow, ++this.numRows);
-
-    // Ensure the Advanced Search container is visible, if this is the first
-    // row being added.
-    if (this.numRows == 1) {
-      this.show();
-
-      // Pre-fill the search terms field with the value from the one on the
-      // toolbar.
-      // For some reason, setting.value here synchronously does not appear to
-      // work.
-      var searchTermsField = document.getElementById("advancedSearch1Textbox");
-      if (searchTermsField)
-        setTimeout(function() { searchTermsField.value = PlacesSearchBox.value; }, 10);
-      this.queries = PlacesOrganizer.getCurrentQueries();
-      return;
-    }
-
-    this.showSearch(this.numRows, searchType);
-    this._updateUIForRowChange();
-  },
-
-  /**
-   * Remove a row from the set of terms
-   * @param   row
-   *          The row to remove. If this is null, the last row will be removed.
-   * If there are no more rows, the query builder will be hidden.
-   */
-  removeRow: function PQB_removeRow(row) {
-    if (!row)
-      row = document.getElementById("advancedSearch" + this.numRows + "Row");
-    row.parentNode.removeChild(row);
-    --this.numRows;
-
-    if (this.numRows < 1) {
-      this.hide();
-
-      // Re-do the original toolbar-search-box search that the user used to
-      // spawn the advanced UI... this effectively "reverts" the UI to the
-      // point it was in before they began monkeying with advanced search.
-      PlacesSearchBox.search(PlacesSearchBox.value);
-      return;
-    }
-
-    this.doSearch();
-    this._updateUIForRowChange();
-  },
-
-  onDateTyped: function PQB_onDateTyped(event, row) {
-    var textbox = document.getElementById("advancedSearch" + row + "TimePicker");
-    var dateString = textbox.value;
-    var dateArr = dateString.split("-");
-    // The date can be split into a range by the '-' character, i.e.
-    // 9/5/05 - 10/2/05.  Unfortunately, dates can also be written like
-    // 9-5-05 - 10-2-05.  Try to parse the date based on how many hyphens
-    // there are.
-    var d0 = null;
-    var d1 = null;
-    // If there are an even number of elements in the date array, try to
-    // parse it as a range of two dates.
-    if ((dateArr.length & 1) == 0) {
-      var mid = dateArr.length / 2;
-      var dateStr0 = dateArr[0];
-      var dateStr1 = dateArr[mid];
-      for (var i = 1; i < mid; ++i) {
-        dateStr0 += "-" + dateArr[i];
-        dateStr1 += "-" + dateArr[i + mid];
-      }
-      d0 = new Date(dateStr0);
-      d1 = new Date(dateStr1);
-    }
-    // If that didn't work, try to parse it as a single date.
-    if (d0 == null || d0 == "Invalid Date") {
-      d0 = new Date(dateString);
-    }
-
-    if (d0 != null && d0 != "Invalid Date") {
-      // Parsing succeeded -- update the calendar.
-      var calendar = document.getElementById("advancedSearch" + row + "Calendar");
-      if (d0.getFullYear() < 2000)
-        d0.setFullYear(2000 + (d0.getFullYear() % 100));
-      if (d1 != null && d1 != "Invalid Date") {
-        if (d1.getFullYear() < 2000)
-          d1.setFullYear(2000 + (d1.getFullYear() % 100));
-        calendar.updateSelection(d0, d1);
-      }
-      else {
-        calendar.updateSelection(d0, d0);
-      }
-
-      // And update the search.
-      this.doSearch();
-    }
-  },
-
-  onCalendarChanged: function PQB_onCalendarChanged(event, row) {
-    var calendar = document.getElementById("advancedSearch" + row + "Calendar");
-    var begin = calendar.beginrange;
-    var end = calendar.endrange;
-
-    // If the calendar doesn't have a begin/end, don't change the textbox.
-    if (begin == null || end == null)
-      return true;
-
-    // If the begin and end are the same day, only fill that into the textbox.
-    var textbox = document.getElementById("advancedSearch" + row + "TimePicker");
-    var beginDate = begin.getDate();
-    var beginMonth = begin.getMonth() + 1;
-    var beginYear = begin.getFullYear();
-    var endDate = end.getDate();
-    var endMonth = end.getMonth() + 1;
-    var endYear = end.getFullYear();
-    if (beginDate == endDate && beginMonth == endMonth && beginYear == endYear) {
-      // Just one date.
-      textbox.value = this._dateService.FormatDate("",
-                                                   this._dateService.dateFormatShort,
-                                                   beginYear,
-                                                   beginMonth,
-                                                   beginDate);
-    }
-    else
-    {
-      // Two dates.
-      var beginStr = this._dateService.FormatDate("",
-                                                   this._dateService.dateFormatShort,
-                                                   beginYear,
-                                                   beginMonth,
-                                                   beginDate);
-      var endStr = this._dateService.FormatDate("",
-                                                this._dateService.dateFormatShort,
-                                                endYear,
-                                                endMonth,
-                                                endDate);
-      textbox.value = beginStr + " - " + endStr;
-    }
-
-    // Update the search.
-    this.doSearch();
-
-    return true;
-  },
-
-  handleTimePickerClick: function PQB_handleTimePickerClick(event, row) {
-    var popup = document.getElementById("advancedSearch" + row + "DatePopup");
-    if (popup.showing)
-      popup.hidePopup();
-    else {
-      var textbox = document.getElementById("advancedSearch" + row + "TimePicker");
-      popup.showPopup(textbox, -1, -1, "popup", "bottomleft", "topleft");
-    }
-  },
-
-  showSearch: function PQB_showSearch(row, values) {
-    for (val in values) {
-      var id = val.replace("_N_", row);
-      var element = document.getElementById(id);
-      if (values[val] || typeof(values[val]) == "string") {
-        if (typeof(values[val]) == "string") {
-          if (values[val] == "date") {
-            // "date" means that the current date should be filled into the
-            // textbox, and the calendar for the row updated.
-            var d = new Date();
-            element.value = this._dateService.FormatDate("",
-                                                         this._dateService.dateFormatShort,
-                                                         d.getFullYear(),
-                                                         d.getMonth() + 1,
-                                                         d.getDate());
-            var calendar = document.getElementById("advancedSearch" + row + "Calendar");
-            calendar.updateSelection(d, d);
-          }
-          else if (element.nodeName == "textbox") {
-            // values[val] is the initial value of the textbox.
-            element.value = values[val];
-          } else {
-            // values[val] is the menuitem which should be selected.
-            var itemId = values[val].replace("_N_", row);
-            var item = document.getElementById(itemId);
-            element.selectedItem = item;
-          }
-        }
-        element.hidden = false;
-      }
-      else {
-        element.hidden = true;
-      }
-    }
-
-    this.doSearch();
-  },
-
-  setKeywordQuery: function PQB_setKeywordQuery(query, prefix) {
-    query.searchTerms += document.getElementById(prefix + "Textbox").value + " ";
-  },
-
-  setLocationQuery: function PQB_setLocationQuery(query, prefix) {
-    var type = document.getElementById(prefix + "LocationMenulist").selectedItem.value;
-    if (type == "onsite") {
-      query.domain = document.getElementById(prefix + "Textbox").value;
-    }
-    else {
-      query.uriIsPrefix = (type == "startswith");
-      var spec = document.getElementById(prefix + "Textbox").value;
-      var ios = Cc["@mozilla.org/network/io-service;1"].
-                getService(Ci.nsIIOService);
-      try {
-        query.uri = ios.newURI(spec, null, null);
-      }
-      catch (e) {
-        // Invalid input can cause newURI to barf, that's OK, tack "http://"
-        // onto the front and try again to see if the user omitted it
-        try {
-          query.uri = ios.newURI("http://" + spec, null, null);
-        }
-        catch (e) {
-          // OK, they have entered something which can never match. This should
-          // not happen.
-        }
-      }
-    }
-  },
-
-  setVisitedQuery: function PQB_setVisitedQuery(query, prefix) {
-    var searchType = document.getElementById(prefix + "TimeMenulist").selectedItem.value;
-    const DAY_MSEC = 86400000;
-    switch (searchType) {
-      case "on":
-        var calendar = document.getElementById(prefix + "Calendar");
-        var begin = calendar.beginrange.getTime();
-        var end = calendar.endrange.getTime();
-        if (begin == end) {
-          end = begin + DAY_MSEC;
-        }
-        query.beginTime = begin * 1000;
-        query.endTime = end * 1000;
-        break;
-      case "before":
-        var calendar = document.getElementById(prefix + "Calendar");
-        var time = calendar.beginrange.getTime();
-        query.endTime = time * 1000;
-        break;
-      case "after":
-        var calendar = document.getElementById(prefix + "Calendar");
-        var time = calendar.endrange.getTime();
-        query.beginTime = time * 1000;
-        break;
-      case "inLast":
-        var textbox = document.getElementById(prefix + "Textbox");
-        var amount = parseInt(textbox.value);
-        amount = amount * DAY_MSEC;
-        var menulist = document.getElementById(prefix + "TimeMenulist2");
-        if (menulist.selectedItem.value == "weeks")
-          amount = amount * 7;
-        else if (menulist.selectedItem.value == "months")
-          amount = amount * 30;
-        var now = new Date();
-        now = now - amount;
-        query.beginTime = now * 1000;
-        break;
-    }
-  },
-
-  doSearch: function PQB_doSearch() {
-    // Create the individual queries.
-    var queryType = document.getElementById("advancedSearchType").selectedItem.value;
-    this.queries = [];
-    if (queryType == "and")
-      this.queries.push(PlacesUtils.history.getNewQuery());
-    var updated = 0;
-    for (var i = 1; updated < this.numRows; ++i) {
-      var prefix = "advancedSearch" + i;
-
-      // The user can remove rows from the middle and start of the list, not
-      // just from the end, so we need to make sure that this row actually
-      // exists before attempting to construct a query for it.
-      var querySubjectElement = document.getElementById(prefix + "Subject");
-      if (querySubjectElement) {
-        // If the queries are being AND-ed, put all the rows in one query.
-        // If they're being OR-ed, add a separate query for each row.
-        var query;
-        if (queryType == "and")
-          query = this.queries[0];
-        else
-          query = PlacesUtils.history.getNewQuery();
-
-        var querySubject = querySubjectElement.value;
-        this._queryBuilders[querySubject](query, prefix);
-
-        if (queryType == "or")
-          this.queries.push(query);
-
-        ++updated;
-      }
-    }
-
-    // Make sure we're getting uri results, not visits
-    this.options = PlacesOrganizer.getCurrentOptions();
-    this.options.resultType = this.options.RESULT_TYPE_URI;
-
-    // XXXben - find some public way of doing this!
-    PlacesOrganizer._content.load(this.queries, this.options);
-
-    // Update the details panel
-    PlacesOrganizer.onContentTreeSelect();
-  },
-#endif
-
   /**
    * Called when a scope button in the scope bar is clicked.
    * @param   aButton
@@ -1804,7 +1286,7 @@ var ViewMenu = {
    * If both aColumnID and aDirection are null, the view will be unsorted.
    */
   setSortColumn: function VM_setSortColumn(aColumn, aDirection) {
-    var result = document.getElementById("placeContent").getResult();
+    var result = document.getElementById("placeContent").result;
     if (!aColumn && !aDirection) {
       result.sortingMode = Ci.nsINavHistoryQueryOptions.SORT_BY_NONE;
       return;
@@ -1842,7 +1324,7 @@ var ViewMenu = {
       lastModified: { key: "LASTMODIFIED", dir: "descending" },
       description:  { key: "ANNOTATION",
                       dir: "ascending",
-                      anno: DESCRIPTION_ANNO }
+                      anno: PlacesUIUtils.DESCRIPTION_ANNO }
     };
 
     // Make sure we have a valid column.
