@@ -57,6 +57,9 @@
 #include "nsGkAtoms.h"
 #include "nsContentUtils.h"
 #include "nsGenericDOMDataNode.h"
+#include "nsClientRect.h"
+#include "nsLayoutUtils.h"
+#include "nsTextFrame.h"
 
 nsresult NS_NewContentIterator(nsIContentIterator** aInstancePtrResult);
 nsresult NS_NewContentSubtreeIterator(nsIContentIterator** aInstancePtrResult);
@@ -235,6 +238,8 @@ NS_IMPL_CYCLE_COLLECTION_CLASS(nsRange)
 NS_IMPL_CYCLE_COLLECTING_ADDREF(nsRange)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(nsRange)
 
+DOMCI_DATA(Range, nsRange)
+
 // QueryInterface implementation for nsRange
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsRange)
   NS_INTERFACE_MAP_ENTRY(nsIDOMRange)
@@ -242,7 +247,7 @@ NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsRange)
   NS_INTERFACE_MAP_ENTRY(nsIDOMNSRange)
   NS_INTERFACE_MAP_ENTRY(nsIMutationObserver)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIRange)
-  NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(Range)
+  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(Range)
 NS_INTERFACE_MAP_END
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsRange)
@@ -310,7 +315,8 @@ void
 nsRange::ContentRemoved(nsIDocument* aDocument,
                         nsIContent* aContainer,
                         nsIContent* aChild,
-                        PRInt32 aIndexInContainer)
+                        PRInt32 aIndexInContainer,
+                        nsIContent* aPreviousSibling)
 {
   NS_ASSERTION(mIsPositioned, "shouldn't be notified if not positioned");
 
@@ -348,6 +354,8 @@ nsRange::ParentChainChanged(nsIContent *aContent)
   NS_ASSERTION(newRoot, "No valid boundary or root found!");
   NS_ASSERTION(newRoot == IsValidBoundary(mEndParent),
                "Start parent and end parent give different root!");
+  // This is safe without holding a strong ref to self as long as the change
+  // of mRoot is the last thing in DoSetRange.
   DoSetRange(mStartParent, mStartOffset, mEndParent, mEndOffset, newRoot);
 }
 
@@ -468,6 +476,8 @@ nsRange::DoSetRange(nsINode* aStartN, PRInt32 aStartOffset,
   mEndParent = aEndN;
   mEndOffset = aEndOffset;
   mIsPositioned = !!mStartParent;
+  // This needs to be the last thing this function does.  See comment
+  // in ParentChainChanged.
   mRoot = aRoot;
 }
 
@@ -491,7 +501,7 @@ IndexOf(nsIDOMNode* aChildNode)
  * nsIRange implementation
  ******************************************************/
 
-nsINode*
+/* virtual */ nsINode*
 nsRange::GetCommonAncestor() const
 {
   return mIsPositioned ?
@@ -499,7 +509,7 @@ nsRange::GetCommonAncestor() const
     nsnull;
 }
 
-void
+/* virtual */ void
 nsRange::Reset()
 {
   DoSetRange(nsnull, 0, nsnull, 0, nsnull);
@@ -509,7 +519,8 @@ nsRange::Reset()
  * public functionality
  ******************************************************/
 
-nsresult nsRange::GetStartContainer(nsIDOMNode** aStartParent)
+NS_IMETHODIMP
+nsRange::GetStartContainer(nsIDOMNode** aStartParent)
 {
   if (!mIsPositioned)
     return NS_ERROR_NOT_INITIALIZED;
@@ -517,7 +528,8 @@ nsresult nsRange::GetStartContainer(nsIDOMNode** aStartParent)
   return CallQueryInterface(mStartParent, aStartParent);
 }
 
-nsresult nsRange::GetStartOffset(PRInt32* aStartOffset)
+NS_IMETHODIMP
+nsRange::GetStartOffset(PRInt32* aStartOffset)
 {
   if (!mIsPositioned)
     return NS_ERROR_NOT_INITIALIZED;
@@ -527,7 +539,8 @@ nsresult nsRange::GetStartOffset(PRInt32* aStartOffset)
   return NS_OK;
 }
 
-nsresult nsRange::GetEndContainer(nsIDOMNode** aEndParent)
+NS_IMETHODIMP
+nsRange::GetEndContainer(nsIDOMNode** aEndParent)
 {
   if (!mIsPositioned)
     return NS_ERROR_NOT_INITIALIZED;
@@ -535,7 +548,8 @@ nsresult nsRange::GetEndContainer(nsIDOMNode** aEndParent)
   return CallQueryInterface(mEndParent, aEndParent);
 }
 
-nsresult nsRange::GetEndOffset(PRInt32* aEndOffset)
+NS_IMETHODIMP
+nsRange::GetEndOffset(PRInt32* aEndOffset)
 {
   if (!mIsPositioned)
     return NS_ERROR_NOT_INITIALIZED;
@@ -545,7 +559,8 @@ nsresult nsRange::GetEndOffset(PRInt32* aEndOffset)
   return NS_OK;
 }
 
-nsresult nsRange::GetCollapsed(PRBool* aIsCollapsed)
+NS_IMETHODIMP
+nsRange::GetCollapsed(PRBool* aIsCollapsed)
 {
   if(mIsDetached)
     return NS_ERROR_DOM_INVALID_STATE_ERR;
@@ -557,7 +572,8 @@ nsresult nsRange::GetCollapsed(PRBool* aIsCollapsed)
   return NS_OK;
 }
 
-nsresult nsRange::GetCommonAncestorContainer(nsIDOMNode** aCommonParent)
+NS_IMETHODIMP
+nsRange::GetCommonAncestorContainer(nsIDOMNode** aCommonParent)
 {
   *aCommonParent = nsnull;
   if(mIsDetached)
@@ -620,7 +636,8 @@ nsINode* nsRange::IsValidBoundary(nsINode* aNode)
   return root;
 }
 
-nsresult nsRange::SetStart(nsIDOMNode* aParent, PRInt32 aOffset)
+NS_IMETHODIMP
+nsRange::SetStart(nsIDOMNode* aParent, PRInt32 aOffset)
 {
   VALIDATE_ACCESS(aParent);
 
@@ -628,7 +645,8 @@ nsresult nsRange::SetStart(nsIDOMNode* aParent, PRInt32 aOffset)
   return SetStart(parent, aOffset);
 }
 
-nsresult nsRange::SetStart(nsINode* aParent, PRInt32 aOffset)
+/* virtual */ nsresult
+nsRange::SetStart(nsINode* aParent, PRInt32 aOffset)
 {
   nsINode* newRoot = IsValidBoundary(aParent);
   NS_ENSURE_TRUE(newRoot, NS_ERROR_DOM_RANGE_INVALID_NODE_TYPE_ERR);
@@ -652,7 +670,8 @@ nsresult nsRange::SetStart(nsINode* aParent, PRInt32 aOffset)
   return NS_OK;
 }
 
-nsresult nsRange::SetStartBefore(nsIDOMNode* aSibling)
+NS_IMETHODIMP
+nsRange::SetStartBefore(nsIDOMNode* aSibling)
 {
   VALIDATE_ACCESS(aSibling);
   
@@ -665,7 +684,8 @@ nsresult nsRange::SetStartBefore(nsIDOMNode* aSibling)
   return SetStart(parent, IndexOf(aSibling));
 }
 
-nsresult nsRange::SetStartAfter(nsIDOMNode* aSibling)
+NS_IMETHODIMP
+nsRange::SetStartAfter(nsIDOMNode* aSibling)
 {
   VALIDATE_ACCESS(aSibling);
 
@@ -678,7 +698,8 @@ nsresult nsRange::SetStartAfter(nsIDOMNode* aSibling)
   return SetStart(nParent, IndexOf(aSibling) + 1);
 }
 
-nsresult nsRange::SetEnd(nsIDOMNode* aParent, PRInt32 aOffset)
+NS_IMETHODIMP
+nsRange::SetEnd(nsIDOMNode* aParent, PRInt32 aOffset)
 {
   VALIDATE_ACCESS(aParent);
 
@@ -687,7 +708,8 @@ nsresult nsRange::SetEnd(nsIDOMNode* aParent, PRInt32 aOffset)
 }
 
 
-nsresult nsRange::SetEnd(nsINode* aParent, PRInt32 aOffset)
+/* virtual */ nsresult
+nsRange::SetEnd(nsINode* aParent, PRInt32 aOffset)
 {
   nsINode* newRoot = IsValidBoundary(aParent);
   NS_ENSURE_TRUE(newRoot, NS_ERROR_DOM_RANGE_INVALID_NODE_TYPE_ERR);
@@ -712,7 +734,8 @@ nsresult nsRange::SetEnd(nsINode* aParent, PRInt32 aOffset)
   return NS_OK;
 }
 
-nsresult nsRange::SetEndBefore(nsIDOMNode* aSibling)
+NS_IMETHODIMP
+nsRange::SetEndBefore(nsIDOMNode* aSibling)
 {
   VALIDATE_ACCESS(aSibling);
   
@@ -725,7 +748,8 @@ nsresult nsRange::SetEndBefore(nsIDOMNode* aSibling)
   return SetEnd(nParent, IndexOf(aSibling));
 }
 
-nsresult nsRange::SetEndAfter(nsIDOMNode* aSibling)
+NS_IMETHODIMP
+nsRange::SetEndAfter(nsIDOMNode* aSibling)
 {
   VALIDATE_ACCESS(aSibling);
   
@@ -738,7 +762,8 @@ nsresult nsRange::SetEndAfter(nsIDOMNode* aSibling)
   return SetEnd(nParent, IndexOf(aSibling) + 1);
 }
 
-nsresult nsRange::Collapse(PRBool aToStart)
+NS_IMETHODIMP
+nsRange::Collapse(PRBool aToStart)
 {
   if(mIsDetached)
     return NS_ERROR_DOM_INVALID_STATE_ERR;
@@ -753,7 +778,8 @@ nsresult nsRange::Collapse(PRBool aToStart)
   return NS_OK;
 }
 
-nsresult nsRange::SelectNode(nsIDOMNode* aN)
+NS_IMETHODIMP
+nsRange::SelectNode(nsIDOMNode* aN)
 {
   VALIDATE_ACCESS(aN);
   
@@ -774,7 +800,8 @@ nsresult nsRange::SelectNode(nsIDOMNode* aN)
   return NS_OK;
 }
 
-nsresult nsRange::SelectNodeContents(nsIDOMNode* aN)
+NS_IMETHODIMP
+nsRange::SelectNodeContents(nsIDOMNode* aN)
 {
   VALIDATE_ACCESS(aN);
 
@@ -864,8 +891,8 @@ RangeSubtreeIterator::Init(nsIDOMRange *aRange)
     PRInt32 startIndex;
     aRange->GetStartOffset(&startIndex);
     nsCOMPtr<nsINode> iNode = do_QueryInterface(node);
-    if (iNode->IsNodeOfType(nsINode::eELEMENT) && 
-        PRInt32(iNode->GetChildCount()) == startIndex) {
+    if (iNode->IsElement() && 
+        PRInt32(iNode->AsElement()->GetChildCount()) == startIndex) {
       mStart = node;
     }
   }
@@ -884,7 +911,7 @@ RangeSubtreeIterator::Init(nsIDOMRange *aRange)
     PRInt32 endIndex;
     aRange->GetEndOffset(&endIndex);
     nsCOMPtr<nsINode> iNode = do_QueryInterface(node);
-    if (iNode->IsNodeOfType(nsINode::eELEMENT) && endIndex == 0) {
+    if (iNode->IsElement() && endIndex == 0) {
       mEnd = node;
     }
   }
@@ -1174,7 +1201,8 @@ static nsresult SplitDataNode(nsIDOMCharacterData* aStartNode,
   return CallQueryInterface(newData, aEndNode);
 }
 
-nsresult PrependChild(nsIDOMNode* aParent, nsIDOMNode* aChild)
+NS_IMETHODIMP
+PrependChild(nsIDOMNode* aParent, nsIDOMNode* aChild)
 {
   nsCOMPtr<nsIDOMNode> first, tmpNode;
   aParent->GetFirstChild(getter_AddRefs(first));
@@ -1340,10 +1368,10 @@ nsresult nsRange::CutContents(nsIDOMDocumentFragment** aFragment)
     if (!handled && (node == endContainer || node == startContainer))
     {
       nsCOMPtr<nsINode> iNode = do_QueryInterface(node);
-      if (iNode && iNode->IsNodeOfType(nsINode::eELEMENT) &&
+      if (iNode && iNode->IsElement() &&
           ((node == endContainer && endOffset == 0) ||
            (node == startContainer &&
-            PRInt32(iNode->GetChildCount()) == startOffset)))
+            PRInt32(iNode->AsElement()->GetChildCount()) == startOffset)))
       {
         if (retval) {
           nsCOMPtr<nsIDOMNode> clone;
@@ -1422,18 +1450,6 @@ nsresult nsRange::CutContents(nsIDOMDocumentFragment** aFragment)
     }
   }
 
-  // XXX_kin: At this point we should be checking for the case
-  // XXX_kin: where we have 2 adjacent text nodes left, each
-  // XXX_kin: containing one of the range end points. The spec
-  // XXX_kin: says the 2 nodes should be merged in that case,
-  // XXX_kin: and to use Normalize() to do the merging, but
-  // XXX_kin: calling Normalize() on the common parent to accomplish
-  // XXX_kin: this might also normalize nodes that are outside the
-  // XXX_kin: range but under the common parent. Need to verify
-  // XXX_kin: with the range commitee members that this was the
-  // XXX_kin: desired behavior. For now we don't merge anything!
-  // XXX ajvincent Filed as https://bugzilla.mozilla.org/show_bug.cgi?id=401276
-
   rv = CollapseRangeAfterDelete(this);
   if (NS_SUCCEEDED(rv) && aFragment) {
     NS_ADDREF(*aFragment = retval);
@@ -1441,12 +1457,14 @@ nsresult nsRange::CutContents(nsIDOMDocumentFragment** aFragment)
   return rv;
 }
 
-nsresult nsRange::DeleteContents()
+NS_IMETHODIMP
+nsRange::DeleteContents()
 {
   return CutContents(nsnull);
 }
 
-nsresult nsRange::ExtractContents(nsIDOMDocumentFragment** aReturn)
+NS_IMETHODIMP
+nsRange::ExtractContents(nsIDOMDocumentFragment** aReturn)
 {
   NS_ENSURE_ARG_POINTER(aReturn);
   return CutContents(aReturn);
@@ -1506,7 +1524,7 @@ nsRange::CompareBoundaryPoints(PRUint16 aHow, nsIDOMRange* aOtherRange,
   return NS_OK;
 }
 
-nsresult
+/* static */ nsresult
 nsRange::CloneParentsBetween(nsIDOMNode *aAncestor,
                              nsIDOMNode *aNode,
                              nsIDOMNode **aClosestAncestor,
@@ -1557,7 +1575,8 @@ nsRange::CloneParentsBetween(nsIDOMNode *aAncestor,
   return NS_OK;
 }
 
-nsresult nsRange::CloneContents(nsIDOMDocumentFragment** aReturn)
+NS_IMETHODIMP
+nsRange::CloneContents(nsIDOMDocumentFragment** aReturn)
 {
   if (IsDetached())
     return NS_ERROR_DOM_INVALID_STATE_ERR;
@@ -1619,10 +1638,11 @@ nsresult nsRange::CloneContents(nsIDOMDocumentFragment** aReturn)
   {
     nsCOMPtr<nsIDOMNode> node(iter.GetCurrentNode());
     nsCOMPtr<nsINode> iNode = do_QueryInterface(node);
-    PRBool deepClone = !(iNode->IsNodeOfType(nsINode::eELEMENT)) ||
+    PRBool deepClone = !iNode->IsElement() ||
                        (!(iNode == mEndParent && mEndOffset == 0) &&
                         !(iNode == mStartParent &&
-                          mStartOffset == PRInt32(iNode->GetChildCount())));
+                          mStartOffset ==
+                            PRInt32(iNode->AsElement()->GetChildCount())));
 
     // Clone the current subtree!
 
@@ -1780,12 +1800,14 @@ NS_IMETHODIMP nsRange::CloneRange(nsIDOMRange** aReturn)
   return rv;
 }
 
-nsresult nsRange::CloneRange(nsIRange** aReturn) const
+/* virtual */ nsresult
+nsRange::CloneRange(nsIRange** aReturn) const
 {
   return DoCloneRange(aReturn);
 }
 
-nsresult nsRange::InsertNode(nsIDOMNode* aN)
+NS_IMETHODIMP
+nsRange::InsertNode(nsIDOMNode* aN)
 {
   VALIDATE_ACCESS(aN);
   
@@ -1804,27 +1826,26 @@ nsresult nsRange::InsertNode(nsIDOMNode* aN)
     res = tStartContainer->GetParentNode(getter_AddRefs(tSCParentNode));
     if(NS_FAILED(res)) return res;
     NS_ENSURE_STATE(tSCParentNode);
-    
-    PRBool isCollapsed;
-    res = GetCollapsed(&isCollapsed);
-    if(NS_FAILED(res)) return res;
 
     PRInt32 tEndOffset;
     GetEndOffset(&tEndOffset);
 
+    nsCOMPtr<nsIDOMNode> tEndContainer;
+    res = this->GetEndContainer(getter_AddRefs(tEndContainer));
+    if(NS_FAILED(res)) return res;
+
     nsCOMPtr<nsIDOMText> secondPart;
     res = startTextNode->SplitText(tStartOffset, getter_AddRefs(secondPart));
     if (NS_FAILED(res)) return res;
-    
-    // SplitText collapses the range; fix that (bug 253609)
-    if (!isCollapsed)
-    {
-      res = SetEnd(secondPart, tEndOffset - tStartOffset);
-      if(NS_FAILED(res)) return res;
-    }
-    
+
     nsCOMPtr<nsIDOMNode> tResultNode;
-    return tSCParentNode->InsertBefore(aN, secondPart, getter_AddRefs(tResultNode));
+    res = tSCParentNode->InsertBefore(aN, secondPart, getter_AddRefs(tResultNode));
+    if (NS_FAILED(res)) return res;
+
+    if (tEndContainer == tStartContainer && tEndOffset != tStartOffset)
+      res = SetEnd(secondPart, tEndOffset - tStartOffset);
+
+    return res;
   }  
 
   nsCOMPtr<nsIDOMNodeList>tChildList;
@@ -1843,7 +1864,8 @@ nsresult nsRange::InsertNode(nsIDOMNode* aN)
   return tStartContainer->InsertBefore(aN, tChildNode, getter_AddRefs(tResultNode));
 }
 
-nsresult nsRange::SurroundContents(nsIDOMNode* aNewParent)
+NS_IMETHODIMP
+nsRange::SurroundContents(nsIDOMNode* aNewParent)
 {
   VALIDATE_ACCESS(aNewParent);
 
@@ -1918,7 +1940,8 @@ nsresult nsRange::SurroundContents(nsIDOMNode* aNewParent)
   return SelectNode(aNewParent);
 }
 
-nsresult nsRange::ToString(nsAString& aReturn)
+NS_IMETHODIMP
+nsRange::ToString(nsAString& aReturn)
 { 
   if(mIsDetached)
     return NS_ERROR_DOM_INVALID_STATE_ERR;
@@ -2011,7 +2034,7 @@ nsresult nsRange::ToString(nsAString& aReturn)
 
 
 
-nsresult
+NS_IMETHODIMP
 nsRange::Detach()
 {
   if(mIsDetached)
@@ -2029,10 +2052,188 @@ NS_IMETHODIMP
 nsRange::CreateContextualFragment(const nsAString& aFragment,
                                   nsIDOMDocumentFragment** aReturn)
 {
-  nsCOMPtr<nsIDOMNode> start = do_QueryInterface(mStartParent);
   if (mIsPositioned) {
-    return nsContentUtils::CreateContextualFragment(start, aFragment, PR_TRUE,
-                                                    aReturn);
+    return nsContentUtils::CreateContextualFragment(mStartParent, aFragment,
+                                                    PR_TRUE, aReturn);
   }
   return NS_ERROR_FAILURE;
 }
+
+static void ExtractRectFromOffset(nsIFrame* aFrame,
+                                  const nsIFrame* aRelativeTo, 
+                                  const PRInt32 aOffset, nsRect* aR, PRBool aKeepLeft)
+{
+  nsPoint point;
+  aFrame->GetPointFromOffset(aOffset, &point);
+
+  point += aFrame->GetOffsetTo(aRelativeTo);
+
+  //given a point.x, extract left or right portion of rect aR
+  //point.x has to be within this rect
+  NS_ASSERTION(aR->x <= point.x && point.x <= aR->XMost(),
+                   "point.x should not be outside of rect r");
+
+  if (aKeepLeft) {
+    aR->width = point.x - aR->x;
+  } else {
+    aR->width = aR->XMost() - point.x;
+    aR->x = point.x;
+  }
+}
+
+static nsresult GetPartialTextRect(nsLayoutUtils::RectCallback* aCallback,
+                                   nsIContent* aContent, PRInt32 aStartOffset, PRInt32 aEndOffset)
+{
+  nsIFrame* frame = aContent->GetPrimaryFrame();
+  if (frame && frame->GetType() == nsGkAtoms::textFrame) {
+    nsTextFrame* textFrame = static_cast<nsTextFrame*>(frame);
+    nsIFrame* relativeTo = nsLayoutUtils::GetContainingBlockForClientRect(textFrame);
+    for (nsTextFrame* f = textFrame; f; f = static_cast<nsTextFrame*>(f->GetNextContinuation())) {
+      PRInt32 fstart = f->GetContentOffset(), fend = f->GetContentEnd();
+      if (fend <= aStartOffset || fstart >= aEndOffset)
+        continue;
+
+      // overlapping with the offset we want
+      f->EnsureTextRun();
+      NS_ENSURE_TRUE(f->GetTextRun(), NS_ERROR_OUT_OF_MEMORY);
+      PRBool rtl = f->GetTextRun()->IsRightToLeft();
+      nsRect r(f->GetOffsetTo(relativeTo), f->GetSize());
+      if (fstart < aStartOffset) {
+        // aStartOffset is within this frame
+        ExtractRectFromOffset(f, relativeTo, aStartOffset, &r, rtl);
+      }
+      if (fend > aEndOffset) {
+        // aEndOffset is in the middle of this frame
+        ExtractRectFromOffset(f, relativeTo, aEndOffset, &r, !rtl);
+      }
+      aCallback->AddRect(r);
+    }
+  }
+  return NS_OK;
+}
+
+static void CollectClientRects(nsLayoutUtils::RectCallback* aCollector, 
+                               nsRange* aRange,
+                               nsINode* aStartParent, PRInt32 aStartOffset,
+                               nsINode* aEndParent, PRInt32 aEndOffset)
+{
+  // Hold strong pointers across the flush
+  nsCOMPtr<nsIDOMNode> startContainer = do_QueryInterface(aStartParent);
+  nsCOMPtr<nsIDOMNode> endContainer = do_QueryInterface(aEndParent);
+
+  // Flush out layout so our frames are up to date.
+  if (!aStartParent->IsInDoc()) {
+    return;
+  }
+
+  aStartParent->GetCurrentDoc()->FlushPendingNotifications(Flush_Layout);
+
+  // Recheck whether we're still in the document
+  if (!aStartParent->IsInDoc()) {
+    return;
+  }
+
+  RangeSubtreeIterator iter;
+
+  nsresult rv = iter.Init(aRange);
+  if (NS_FAILED(rv)) return;
+
+  if (iter.IsDone()) {
+    // the range is collapsed, only continue if the cursor is in a text node
+    nsCOMPtr<nsIContent> content = do_QueryInterface(aStartParent);
+    if (content && content->IsNodeOfType(nsINode::eTEXT)) {
+      nsIFrame* frame = content->GetPrimaryFrame();
+      if (frame && frame->GetType() == nsGkAtoms::textFrame) {
+        nsTextFrame* textFrame = static_cast<nsTextFrame*>(frame);
+        PRInt32 outOffset;
+        nsIFrame* outFrame;
+        textFrame->GetChildFrameContainingOffset(aStartOffset, PR_FALSE, 
+          &outOffset, &outFrame);
+        if (outFrame) {
+           nsIFrame* relativeTo = 
+             nsLayoutUtils::GetContainingBlockForClientRect(outFrame);
+           nsRect r(outFrame->GetOffsetTo(relativeTo), outFrame->GetSize());
+           ExtractRectFromOffset(outFrame, relativeTo, aStartOffset, &r, PR_FALSE);
+           r.width = 0;
+           aCollector->AddRect(r);
+        }
+      }
+    }
+    return;
+  }
+
+  do {
+    nsCOMPtr<nsIDOMNode> node(iter.GetCurrentNode());
+    iter.Next();
+    nsCOMPtr<nsIContent> content = do_QueryInterface(node);
+    if (!content)
+      continue;
+    if (content->IsNodeOfType(nsINode::eTEXT)) {
+       if (node == startContainer) {
+         PRInt32 offset = startContainer == endContainer ? 
+           aEndOffset : content->GetText()->GetLength();
+         GetPartialTextRect(aCollector, content, aStartOffset, offset);
+         continue;
+       } else if (node == endContainer) {
+         GetPartialTextRect(aCollector, content, 0, aEndOffset);
+         continue;
+       }
+    }
+
+    nsIFrame* frame = content->GetPrimaryFrame();
+    if (frame) {
+      nsLayoutUtils::GetAllInFlowRects(frame,
+        nsLayoutUtils::GetContainingBlockForClientRect(frame), aCollector);
+    }
+  } while (!iter.IsDone());
+}
+
+NS_IMETHODIMP
+nsRange::GetBoundingClientRect(nsIDOMClientRect** aResult)
+{
+  *aResult = nsnull;
+
+  // Weak ref, since we addref it below
+  nsClientRect* rect = new nsClientRect();
+  if (!rect)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(*aResult = rect);
+
+  if (!mStartParent)
+    return NS_OK;
+
+  nsLayoutUtils::RectAccumulator accumulator;
+  
+  CollectClientRects(&accumulator, this, mStartParent, mStartOffset, 
+    mEndParent, mEndOffset);
+
+  nsRect r = accumulator.mResultRect.IsEmpty() ? accumulator.mFirstRect : 
+    accumulator.mResultRect;
+  rect->SetLayoutRect(r);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsRange::GetClientRects(nsIDOMClientRectList** aResult)
+{
+  *aResult = nsnull;
+
+  if (!mStartParent)
+    return NS_OK;
+
+  nsRefPtr<nsClientRectList> rectList = new nsClientRectList();
+  if (!rectList)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  nsLayoutUtils::RectListBuilder builder(rectList);
+
+  CollectClientRects(&builder, this, mStartParent, mStartOffset, 
+    mEndParent, mEndOffset);
+
+  if (NS_FAILED(builder.mRV))
+    return builder.mRV;
+  rectList.forget(aResult);
+  return NS_OK;
+}
+
