@@ -109,8 +109,8 @@ The following result in state transitions.
 
 Shutdown()
   Clean up any resources the nsDecoderStateMachine owns.
-Decode()
-  Start decoding media data.
+Play()
+  Start decoding and playback of media data.
 Buffer
   This is not user initiated. It occurs when the
   available data in the stream drops below a certain point.
@@ -130,13 +130,13 @@ DECODING_METADATA
   |---------------->----->------------------------|        v
 DECODING             |          |  |              |        |
   ^                  v Seek(t)  |  |              |        |
-  |         Decode() |          v  |              |        |
+  |         Play()   |          v  |              |        |
   ^-----------<----SEEKING      |  v Complete     v        v
   |                  |          |  |              |        |
   |                  |          |  COMPLETED    SHUTDOWN-<-|
   ^                  ^          |  |Shutdown()    |
   |                  |          |  >-------->-----^
-  |         Decode() |Seek(t)   |Buffer()         |
+  |          Play()  |Seek(t)   |Buffer()         |
   -----------<--------<-------BUFFERING           |
                                 |                 ^
                                 v Shutdown()      |
@@ -261,14 +261,14 @@ public:
 
   // Functions used by assertions to ensure we're calling things
   // on the appropriate threads.
-  virtual PRBool OnDecodeThread() = 0;
+  virtual PRBool OnDecodeThread() const = 0;
 
   virtual nsHTMLMediaElement::NextFrameStatus GetNextFrameStatus() = 0;
 
   // Cause state transitions. These methods obtain the decoder monitor
   // to synchronise the change of state, and to notify other threads
   // that the state has changed.
-  virtual void Decode() = 0;
+  virtual void Play() = 0;
 
   // Seeks to aTime in seconds
   virtual void Seek(double aTime) = 0;
@@ -276,7 +276,7 @@ public:
   // Returns the current playback position in seconds.
   // Called from the main thread to get the current frame time. The decoder
   // monitor must be obtained before calling this.
-  virtual double GetCurrentTime() = 0;
+  virtual double GetCurrentTime() const = 0;
 
   // Clear the flag indicating that a playback position change event
   // is currently queued. This is called from the main thread and must
@@ -304,7 +304,9 @@ public:
   // the main thread.
   virtual void StartBuffering() = 0;
 
-  virtual void NotifyDataExhausted() = 0;
+  // Sets the current size of the framebuffer used in MozAudioAvailable events.
+  // Called on the state machine thread and the main thread.
+  virtual void SetFrameBufferLength(PRUint32 aLength) = 0;
 };
 
 class nsBuiltinDecoder : public nsMediaDecoder
@@ -369,10 +371,6 @@ class nsBuiltinDecoder : public nsMediaDecoder
   // from the resource.
   void NotifyBytesConsumed(PRInt64 aBytes);
 
-  void NotifyDataExhausted() {
-    mDecoderStateMachine->NotifyDataExhausted();
-  }
-
   // Called when the video file has completed downloading.
   // Call on the main thread only.
   void ResourceLoaded();
@@ -429,7 +427,7 @@ class nsBuiltinDecoder : public nsMediaDecoder
     return IsCurrentThread(mStateMachineThread);
   }
 
-  PRBool OnDecodeThread() {
+  PRBool OnDecodeThread() const {
     return mDecoderStateMachine->OnDecodeThread();
   }
 
@@ -442,13 +440,20 @@ class nsBuiltinDecoder : public nsMediaDecoder
   // Constructs the time ranges representing what segments of the media
   // are buffered and playable.
   virtual nsresult GetBuffered(nsTimeRanges* aBuffered) {
-    return mDecoderStateMachine->GetBuffered(aBuffered);
+    if (mDecoderStateMachine) {
+      return mDecoderStateMachine->GetBuffered(aBuffered);
+    }
+    return NS_ERROR_FAILURE;
   }
 
   virtual void NotifyDataArrived(const char* aBuffer, PRUint32 aLength, PRUint32 aOffset) {
     return mDecoderStateMachine->NotifyDataArrived(aBuffer, aLength, aOffset);
   }
 
+  // Sets the length of the framebuffer used in MozAudioAvailable events.
+  // The new size must be between 512 and 16384.
+  virtual nsresult RequestFrameBufferLength(PRUint32 aLength);
+
  public:
   // Return the current state. Can be called on any thread. If called from
   // a non-main thread, the decoder monitor must be held.
@@ -494,8 +499,7 @@ class nsBuiltinDecoder : public nsMediaDecoder
   // Called when the metadata from the media file has been read.
   // Call on the main thread only.
   void MetadataLoaded(PRUint32 aChannels,
-                      PRUint32 aRate,
-                      PRUint32 aFrameBufferLength);
+                      PRUint32 aRate);
 
   // Called when the first frame has been loaded.
   // Call on the main thread only.
