@@ -48,7 +48,6 @@
 #include "jsdbgapi.h"
 #endif
 #include "nsAtomicRefcnt.h"
-#include "nsAutoLock.h"
 #include "nsAXPCNativeCallContext.h"
 #include "nsContentUtils.h"
 #include "nsDOMClassInfo.h"
@@ -70,6 +69,8 @@
 #include "nsDOMWorkerTimeout.h"
 #include "nsDOMWorkerXHR.h"
 
+using namespace mozilla;
+
 class TestComponentThreadsafetyRunnable : public nsIRunnable
 {
 public:
@@ -1351,7 +1352,7 @@ nsDOMWorker::nsDOMWorker(nsDOMWorker* aParent,
 : mParent(aParent),
   mParentWN(aParentWN),
   mPrivilegeModel(aPrivilegeModel),
-  mLock(nsnull),
+  mLock("nsDOMWorker.mLock"),
   mInnerScope(nsnull),
   mGlobal(NULL),
   mNextTimeoutId(0),
@@ -1375,10 +1376,6 @@ nsDOMWorker::~nsDOMWorker()
     mPool->NoteDyingWorker(this);
   }
 
-  if (mLock) {
-    nsAutoLock::DestroyLock(mLock);
-  }
-
   NS_ASSERTION(!mFeatures.Length(), "Live features!");
   NS_ASSERTION(!mQueuedRunnables.Length(), "Events that never ran!");
 
@@ -1484,15 +1481,29 @@ NS_INTERFACE_MAP_END
 
 NS_IMETHODIMP
 nsDOMWorker::PreCreate(nsISupports* aObject,
-                       JSContext* /* aCx */,
+                       JSContext* aCx,
                        JSObject* /* aPlannedParent */,
-                       JSObject** /* aParent */)
+                       JSObject** aParent)
 {
   nsCOMPtr<nsIWorker> iworker(do_QueryInterface(aObject));
-  if (iworker && static_cast<nsDOMWorker *>(iworker.get())->IsPrivileged()) {
-    return NS_SUCCESS_CHROME_ACCESS_ONLY;
+  NS_ENSURE_TRUE(iworker, NS_ERROR_UNEXPECTED);
+
+  nsCOMPtr<nsIXPConnectWrappedNative> wrappedNative;
+  {
+    MutexAutoLock lock(mLock);
+    wrappedNative = mWrappedNative;
   }
-  return NS_OK;
+
+  // Don't allow XPConnect to create multiple WrappedNatives for this object.
+  if (wrappedNative) {
+    JSObject* object;
+    nsresult rv = wrappedNative->GetJSObject(&object);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    *aParent = JS_GetParent(aCx, object);
+  }
+
+  return IsPrivileged() ? NS_SUCCESS_CHROME_ACCESS_ONLY : NS_OK;
 }
 
 NS_IMETHODIMP
@@ -1500,7 +1511,7 @@ nsDOMWorker::PostCreate(nsIXPConnectWrappedNative* aWrapper,
                         JSContext* /* aCx */,
                         JSObject* /* aObj */)
 {
-  nsAutoLock lock(mLock);
+  MutexAutoLock lock(mLock);
   mWrappedNative = aWrapper;
   return NS_OK;
 }
@@ -1514,7 +1525,7 @@ nsDOMWorker::Trace(nsIXPConnectWrappedNative* /* aWrapper */,
 
   PRBool canceled = PR_FALSE;
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     canceled = mStatus == eKilled;
   }
 
@@ -1537,7 +1548,7 @@ nsDOMWorker::Finalize(nsIXPConnectWrappedNative* /* aWrapper */,
 
   // Clear our wrapped native now that it has died.
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     mWrappedNative = nsnull;
   }
 
@@ -1672,9 +1683,6 @@ nsDOMWorker::InitializeInternal(nsIScriptGlobalObject* aOwner,
     NS_ASSERTION(mPrincipal, "Should have set the principal!");
   }
 
-  mLock = nsAutoLock::NewLock("nsDOMWorker::mLock");
-  NS_ENSURE_TRUE(mLock, NS_ERROR_OUT_OF_MEMORY);
-
   NS_ASSERTION(!mGlobal, "Already got a global?!");
 
   nsCOMPtr<nsIXPConnectJSObjectHolder> thisWrapped;
@@ -1733,7 +1741,7 @@ nsDOMWorker::Cancel()
 
   PRBool enforceTimeout = PR_FALSE;
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
 
     NS_ASSERTION(mStatus != eCanceled, "Canceled more than once?!");
 
@@ -1795,7 +1803,7 @@ nsDOMWorker::Kill()
   PRUint32 count, index;
   nsAutoTArray<nsRefPtr<nsDOMWorkerFeature>, 20> features;
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
 
     if (mStatus == eKilled) {
       NS_ASSERTION(mFeatures.Length() == 0, "Features added after killed!");
@@ -1844,7 +1852,7 @@ nsDOMWorker::Suspend()
 
   PRBool shouldSuspendFeatures;
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     NS_ASSERTION(!mSuspended, "Suspended more than once!");
     shouldSuspendFeatures = !mSuspended;
     mSuspended = PR_TRUE;
@@ -1862,7 +1870,7 @@ nsDOMWorker::Resume()
 
   PRBool shouldResumeFeatures;
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
 #ifdef DEBUG
     // Should only have a mismatch if GC or Cancel happened while suspended.
     if (!mSuspended) {
@@ -1890,7 +1898,7 @@ nsDOMWorker::Resume()
 PRBool
 nsDOMWorker::IsCanceled()
 {
-  nsAutoLock lock(mLock);
+  MutexAutoLock lock(mLock);
   return IsCanceledNoLock();
 }
 
@@ -1924,14 +1932,14 @@ nsDOMWorker::IsCanceledNoLock()
 PRBool
 nsDOMWorker::IsClosing()
 {
-  nsAutoLock lock(mLock);
+  MutexAutoLock lock(mLock);
   return mStatus != eRunning;
 }
 
 PRBool
 nsDOMWorker::IsSuspended()
 {
-  nsAutoLock lock(mLock);
+  MutexAutoLock lock(mLock);
   return mSuspended;
 }
 
@@ -2203,7 +2211,7 @@ nsDOMWorker::GetWrappedNative()
 {
   nsCOMPtr<nsIXPConnectWrappedNative> wrappedNative;
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     wrappedNative = mWrappedNative;
   }
   return wrappedNative.forget();
@@ -2220,7 +2228,7 @@ nsDOMWorker::AddFeature(nsDOMWorkerFeature* aFeature,
     // aCx may be null.
     JSAutoSuspendRequest asr(aCx);
 
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
 
     if (mStatus == eKilled) {
       // No features may be added after we've been canceled. Sorry.
@@ -2253,7 +2261,7 @@ nsDOMWorker::RemoveFeature(nsDOMWorkerFeature* aFeature,
     // aCx may be null.
     JSAutoSuspendRequest asr(aCx);
 
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
 
 #ifdef DEBUG
     PRBool removed =
@@ -2270,7 +2278,7 @@ nsDOMWorker::CancelTimeoutWithId(PRUint32 aId)
 {
   nsRefPtr<nsDOMWorkerFeature> foundFeature;
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     PRUint32 count = mFeatures.Length();
     for (PRUint32 index = 0; index < count; index++) {
       nsDOMWorkerFeature*& feature = mFeatures[index];
@@ -2293,7 +2301,7 @@ nsDOMWorker::SuspendFeatures()
 {
   nsAutoTArray<nsRefPtr<nsDOMWorkerFeature>, 20> features;
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
 
     // We don't really have to worry about overflow here because the only way
     // to do this is through recursive script loading, which uses the stack. We
@@ -2322,7 +2330,7 @@ nsDOMWorker::ResumeFeatures()
 {
   nsAutoTArray<nsRefPtr<nsDOMWorkerFeature>, 20> features;
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
 
     NS_ASSERTION(mFeatureSuspendDepth > 0, "Shouldn't happen!");
     if (--mFeatureSuspendDepth != 0) {
@@ -2383,7 +2391,7 @@ nsDOMWorker::FireCloseRunnable(PRIntervalTime aTimeoutInterval,
   // to do is unblock the waiting thread.
   PRBool wakeUp;
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     NS_ASSERTION(mExpirationTime == 0,
                  "Close runnable should not be scheduled already!");
 
@@ -2396,7 +2404,7 @@ nsDOMWorker::FireCloseRunnable(PRIntervalTime aTimeoutInterval,
   }
 
   if (wakeUp) {
-    nsAutoMonitor mon(mPool->Monitor());
+    MonitorAutoEnter mon(mPool->GetMonitor());
     mon.NotifyAll();
   }
 
@@ -2430,7 +2438,7 @@ nsresult
 nsDOMWorker::Close()
 {
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     NS_ASSERTION(mStatus != eKilled, "This should be impossible!");
     if (mStatus != eRunning) {
       return NS_OK;
@@ -2448,7 +2456,7 @@ nsresult
 nsDOMWorker::TerminateInternal(PRBool aFromFinalize)
 {
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
 #ifdef DEBUG
     if (!aFromFinalize) {
       NS_ASSERTION(mStatus != eCanceled, "Shouldn't be able to get here!");
@@ -2495,7 +2503,7 @@ void
 nsDOMWorker::SetExpirationTime(PRIntervalTime aExpirationTime)
 {
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
 
     NS_ASSERTION(mStatus != eRunning && mStatus != eKilled, "Bad status!");
     NS_ASSERTION(!mExpirationTime || mExpirationTime == PR_INTERVAL_NO_TIMEOUT,
@@ -2509,7 +2517,7 @@ nsDOMWorker::SetExpirationTime(PRIntervalTime aExpirationTime)
 PRIntervalTime
 nsDOMWorker::GetExpirationTime()
 {
-  nsAutoLock lock(mLock);
+  MutexAutoLock lock(mLock);
   return mExpirationTime;
 }
 #endif
@@ -2594,7 +2602,7 @@ nsDOMWorker::DispatchEvent(nsIDOMEvent* aEvent,
                            PRBool* _retval)
 {
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     if (IsCanceledNoLock()) {
       return NS_OK;
     }
@@ -2635,7 +2643,7 @@ NS_IMETHODIMP
 nsDOMWorker::PostMessage(/* JSObject aMessage */)
 {
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     // There's no reason to dispatch this message after the close handler has
     // been triggered since it will never be allowed to run.
     if (mStatus != eRunning) {
