@@ -42,10 +42,10 @@
 
 // Other includes
 #include "jsapi.h"
+#include "mozilla/Monitor.h"
 #include "nsCOMPtr.h"
 #include "nsStringGlue.h"
 #include "nsTArray.h"
-#include "prmon.h"
 
 class nsDOMWorker;
 class nsIDocument;
@@ -55,6 +55,8 @@ class nsIScriptGlobalObject;
 
 class nsDOMWorkerPool
 {
+  typedef mozilla::Monitor Monitor;
+
 public:
   nsDOMWorkerPool(nsIScriptGlobalObject* aGlobalObject,
                   nsIDocument* aDocument);
@@ -79,7 +81,7 @@ public:
   nsresult NoteWorker(nsDOMWorker* aWorker);
   void NoteDyingWorker(nsDOMWorker* aWorker);
 
-  PRMonitor* Monitor() {
+  Monitor& GetMonitor() {
     return mMonitor;
   }
 
@@ -105,7 +107,7 @@ private:
   nsTArray<nsDOMWorker*> mWorkers;
 
   // Monitor for suspending and resuming workers.
-  PRMonitor* mMonitor;
+  Monitor mMonitor;
 
   PRPackedBool mCanceled;
   PRPackedBool mSuspended;
