@@ -119,12 +119,16 @@ nsTextServicesDocument::~nsTextServicesDocument()
   ClearOffsetTable(&mOffsetTable);
 }
 
+#define TS_ATOM(name_, value_) NS_STATIC_ATOM_BUFFER(name_##_buffer, value_)
+#include "nsTSAtomList.h"
+#undef TS_ATOM
+
 /* static */
 void
 nsTextServicesDocument::RegisterAtoms()
 {
   static const nsStaticAtom ts_atoms[] = {
-#define TS_ATOM(name_, value_) { value_, &name_ },
+#define TS_ATOM(name_, value_) NS_STATIC_ATOM(name_##_buffer, &name_),
 #include "nsTSAtomList.h"
 #undef TS_ATOM
   };
@@ -165,8 +169,7 @@ nsTextServicesDocument::InitWithEditor(nsIEditor *aEditor)
   nsCOMPtr<nsISelectionController> selCon;
   nsCOMPtr<nsIDOMDocument> doc;
 
-  if (!aEditor)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aEditor, NS_ERROR_NULL_POINTER);
 
   LOCK_DOC(this);
 
@@ -242,12 +245,10 @@ nsTextServicesDocument::InitWithEditor(nsIEditor *aEditor)
 NS_IMETHODIMP 
 nsTextServicesDocument::GetDocument(nsIDOMDocument **aDoc)
 {
-  if (!aDoc)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aDoc, NS_ERROR_NULL_POINTER);
 
   *aDoc = nsnull; // init out param
-  if (!mDOMDocument)
-    return NS_ERROR_NOT_INITIALIZED;
+  NS_ENSURE_TRUE(mDOMDocument, NS_ERROR_NOT_INITIALIZED);
 
   *aDoc = mDOMDocument;
   NS_ADDREF(*aDoc);
@@ -477,13 +478,11 @@ nsTextServicesDocument::GetCurrentTextBlock(nsString *aStr)
 {
   nsresult result;
 
-  if (!aStr)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aStr, NS_ERROR_NULL_POINTER);
 
   aStr->Truncate();
 
-  if (!mIterator)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(mIterator, NS_ERROR_FAILURE);
 
   LOCK_DOC(this);
 
@@ -538,8 +537,7 @@ nsTextServicesDocument::LastSelectedBlock(TSDBlockSelectionStatus *aSelStatus,
 {
   nsresult result = NS_OK;
 
-  if (!aSelStatus || !aSelOffset || !aSelLength)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aSelStatus && aSelOffset && aSelLength, NS_ERROR_NULL_POINTER);
 
   LOCK_DOC(this);
 
@@ -1011,8 +1009,7 @@ nsTextServicesDocument::PrevBlock()
 {
   nsresult result = NS_OK;
 
-  if (!mIterator)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(mIterator, NS_ERROR_FAILURE);
 
   LOCK_DOC(this);
 
@@ -1083,8 +1080,7 @@ nsTextServicesDocument::NextBlock()
 {
   nsresult result = NS_OK;
 
-  if (!mIterator)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(mIterator, NS_ERROR_FAILURE);
 
   LOCK_DOC(this);
 
@@ -1161,13 +1157,11 @@ nsTextServicesDocument::NextBlock()
 NS_IMETHODIMP
 nsTextServicesDocument::IsDone(PRBool *aIsDone)
 {
-  if (!aIsDone)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aIsDone, NS_ERROR_NULL_POINTER);
 
   *aIsDone = PR_FALSE;
 
-  if (!mIterator)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(mIterator, NS_ERROR_FAILURE);
 
   LOCK_DOC(this);
 
@@ -1183,8 +1177,7 @@ nsTextServicesDocument::SetSelection(PRInt32 aOffset, PRInt32 aLength)
 {
   nsresult result;
 
-  if (!mSelCon || aOffset < 0 || aLength < 0)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(mSelCon && aOffset >= 0 && aLength >= 0, NS_ERROR_FAILURE);
 
   LOCK_DOC(this);
 
@@ -1204,14 +1197,14 @@ nsTextServicesDocument::ScrollSelectionIntoView()
 {
   nsresult result;
 
-  if (!mSelCon)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(mSelCon, NS_ERROR_FAILURE);
 
   LOCK_DOC(this);
 
   // After ScrollSelectionIntoView(), the pending notifications might be flushed
   // and PresShell/PresContext/Frames may be dead. See bug 418470.
-  result = mSelCon->ScrollSelectionIntoView(nsISelectionController::SELECTION_NORMAL, nsISelectionController::SELECTION_FOCUS_REGION, PR_TRUE);
+  result = mSelCon->ScrollSelectionIntoView(nsISelectionController::SELECTION_NORMAL, nsISelectionController::SELECTION_FOCUS_REGION,
+                                            nsISelectionController::SCROLL_SYNCHRONOUS);
 
   UNLOCK_DOC(this);
 
@@ -1530,8 +1523,7 @@ nsTextServicesDocument::InsertText(const nsString *aText)
   if (!editor || !SelectionIsValid())
     return NS_ERROR_FAILURE;
 
-  if (!aText)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aText, NS_ERROR_NULL_POINTER);
 
   // If the selection is not collapsed, we need to save
   // off the selection offsets so we can restore the
@@ -1551,8 +1543,7 @@ nsTextServicesDocument::InsertText(const nsString *aText)
 
     result = SetSelection(mSelStartOffset, 0);
 
-    if (NS_FAILED(result))
-      return result;
+    NS_ENSURE_SUCCESS(result, result);
   }
 
 
@@ -1807,8 +1798,7 @@ nsTextServicesDocument::DidInsertNode(nsIDOMNode *aNode,
 NS_IMETHODIMP
 nsTextServicesDocument::DidDeleteNode(nsIDOMNode *aChild, nsresult aResult)
 {
-  if (NS_FAILED(aResult))
-    return NS_OK;
+  NS_ENSURE_SUCCESS(aResult, NS_OK);
 
   NS_ENSURE_TRUE(mIterator, NS_ERROR_FAILURE);
 
@@ -1819,8 +1809,8 @@ nsTextServicesDocument::DidDeleteNode(nsIDOMNode *aChild, nsresult aResult)
 
   LOCK_DOC(this);
 
-  PRInt32 nodeIndex, tcount;
-  PRBool hasEntry;
+  PRInt32 nodeIndex = 0;
+  PRBool hasEntry = PR_FALSE;
   OffsetEntry *entry;
 
   nsresult result = NodeHasOffsetEntry(&mOffsetTable, aChild, &hasEntry, &nodeIndex);
@@ -1851,10 +1841,10 @@ nsTextServicesDocument::DidDeleteNode(nsIDOMNode *aChild, nsresult aResult)
     // next valid text node in the offset table, and if there
     // wasn't a next, it would've set mIteratorStatus to eIsDone.
 
-    NS_ASSERTION(0, "DeleteNode called for current iterator node."); 
+    NS_ERROR("DeleteNode called for current iterator node."); 
   }
 
-  tcount = mOffsetTable.Length();
+  PRInt32 tcount = mOffsetTable.Length();
 
   while (nodeIndex < tcount)
   {
@@ -1898,8 +1888,7 @@ nsTextServicesDocument::DidJoinNodes(nsIDOMNode  *aLeftNode,
                                      nsIDOMNode  *aParent,
                                      nsresult     aResult)
 {
-  if (NS_FAILED(aResult))
-    return NS_OK;
+  NS_ENSURE_SUCCESS(aResult, NS_OK);
 
   PRInt32 i;
   PRUint16 type;
@@ -1914,36 +1903,35 @@ nsTextServicesDocument::DidJoinNodes(nsIDOMNode  *aLeftNode,
 
   result = aLeftNode->GetNodeType(&type);
 
-  if (NS_FAILED(result))
-    return PR_FALSE;
+  NS_ENSURE_SUCCESS(result, PR_FALSE);
 
   if (nsIDOMNode::TEXT_NODE != type)
   {
-    NS_ASSERTION(0, "JoinNode called with a non-text left node!");
+    NS_ERROR("JoinNode called with a non-text left node!");
     return NS_ERROR_FAILURE;
   }
 
   result = aRightNode->GetNodeType(&type);
 
-  if (NS_FAILED(result))
-    return PR_FALSE;
+  NS_ENSURE_SUCCESS(result, PR_FALSE);
 
   if (nsIDOMNode::TEXT_NODE != type)
   {
-    NS_ASSERTION(0, "JoinNode called with a non-text right node!");
+    NS_ERROR("JoinNode called with a non-text right node!");
     return NS_ERROR_FAILURE;
   }
 
   // Note: The editor merges the contents of the left node into the
   //       contents of the right.
 
-  PRInt32 leftIndex, rightIndex;
-  PRBool leftHasEntry, rightHasEntry;
+  PRInt32 leftIndex = 0;
+  PRInt32 rightIndex = 0;
+  PRBool leftHasEntry = PR_FALSE;
+  PRBool rightHasEntry = PR_FALSE;
 
   result = NodeHasOffsetEntry(&mOffsetTable, aLeftNode, &leftHasEntry, &leftIndex);
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   if (!leftHasEntry)
   {
@@ -1954,8 +1942,7 @@ nsTextServicesDocument::DidJoinNodes(nsIDOMNode  *aLeftNode,
 
   result = NodeHasOffsetEntry(&mOffsetTable, aRightNode, &rightHasEntry, &rightIndex);
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   if (!rightHasEntry)
   {
@@ -2038,8 +2025,7 @@ nsTextServicesDocument::CreateContentIterator(nsIDOMRange *aRange, nsIContentIte
 {
   nsresult result;
 
-  if (!aRange || !aIterator)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aRange && aIterator, NS_ERROR_NULL_POINTER);
 
   *aIterator = 0;
 
@@ -2057,8 +2043,7 @@ nsTextServicesDocument::CreateContentIterator(nsIDOMRange *aRange, nsIContentIte
   }
   NS_ENSURE_SUCCESS(result, result);
 
-  if (!*aIterator)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(*aIterator, NS_ERROR_NULL_POINTER);
 
   result = (*aIterator)->Init(aRange);
 
@@ -2077,13 +2062,11 @@ nsTextServicesDocument::GetDocumentContentRootNode(nsIDOMNode **aNode)
 {
   nsresult result;
 
-  if (!aNode)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aNode, NS_ERROR_NULL_POINTER);
 
   *aNode = 0;
 
-  if (!mDOMDocument)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(mDOMDocument, NS_ERROR_FAILURE);
 
   nsCOMPtr<nsIDOMHTMLDocument> htmlDoc = do_QueryInterface(mDOMDocument);
 
@@ -2095,11 +2078,9 @@ nsTextServicesDocument::GetDocumentContentRootNode(nsIDOMNode **aNode)
 
     result = htmlDoc->GetBody(getter_AddRefs(bodyElement));
 
-    if (NS_FAILED(result))
-      return result;
+    NS_ENSURE_SUCCESS(result, result);
 
-    if (!bodyElement)
-      return NS_ERROR_FAILURE;
+    NS_ENSURE_TRUE(bodyElement, NS_ERROR_FAILURE);
 
     result = bodyElement->QueryInterface(NS_GET_IID(nsIDOMNode), (void **)aNode);
   }
@@ -2111,11 +2092,9 @@ nsTextServicesDocument::GetDocumentContentRootNode(nsIDOMNode **aNode)
 
     result = mDOMDocument->GetDocumentElement(getter_AddRefs(docElement));
 
-    if (NS_FAILED(result))
-      return result;
+    NS_ENSURE_SUCCESS(result, result);
 
-    if (!docElement)
-      return NS_ERROR_FAILURE;
+    NS_ENSURE_TRUE(docElement, NS_ERROR_FAILURE);
 
     result = docElement->QueryInterface(NS_GET_IID(nsIDOMNode), (void **)aNode);
   }
@@ -2128,8 +2107,7 @@ nsTextServicesDocument::CreateDocumentContentRange(nsIDOMRange **aRange)
 {
   nsresult result;
 
-  if (!aRange)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aRange, NS_ERROR_NULL_POINTER);
 
   *aRange = 0;
 
@@ -2137,18 +2115,14 @@ nsTextServicesDocument::CreateDocumentContentRange(nsIDOMRange **aRange)
 
   result = GetDocumentContentRootNode(getter_AddRefs(node));
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
-  if (!node)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(node, NS_ERROR_NULL_POINTER);
 
   result = CallCreateInstance("@mozilla.org/content/range;1", aRange);
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
-  if (!*aRange)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(*aRange, NS_ERROR_NULL_POINTER);
 
   result = (*aRange)->SelectNodeContents(node);
 
@@ -2167,8 +2141,7 @@ nsTextServicesDocument::CreateDocumentContentRootToNodeOffsetRange(nsIDOMNode *a
 {
   nsresult result;
 
-  if (!aParent || !aRange)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aParent && aRange, NS_ERROR_NULL_POINTER);
 
   *aRange = 0;
 
@@ -2181,11 +2154,9 @@ nsTextServicesDocument::CreateDocumentContentRootToNodeOffsetRange(nsIDOMNode *a
 
   result = GetDocumentContentRootNode(getter_AddRefs(bodyNode));
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
-  if (!bodyNode)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(bodyNode, NS_ERROR_NULL_POINTER);
 
   nsCOMPtr<nsIDOMNode> startNode;
   nsCOMPtr<nsIDOMNode> endNode;
@@ -2216,26 +2187,22 @@ nsTextServicesDocument::CreateDocumentContentRootToNodeOffsetRange(nsIDOMNode *a
 
     result = bodyNode->GetChildNodes(getter_AddRefs(nodeList));
 
-    if (NS_FAILED(result))
-      return NS_ERROR_FAILURE;
+    NS_ENSURE_SUCCESS(result, NS_ERROR_FAILURE);
 
     if (nodeList)
     {
       result = nodeList->GetLength(&nodeListLength);
 
-      if (NS_FAILED(result))
-        return NS_ERROR_FAILURE;
+      NS_ENSURE_SUCCESS(result, NS_ERROR_FAILURE);
 
       endOffset = (PRInt32)nodeListLength;
     }
   }
 
   result = CallCreateInstance("@mozilla.org/content/range;1", aRange);
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
-  if (!*aRange)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(*aRange, NS_ERROR_NULL_POINTER);
 
   result = (*aRange)->SetStart(startNode, startOffset);
 
@@ -2256,15 +2223,13 @@ nsTextServicesDocument::CreateDocumentContentIterator(nsIContentIterator **aIter
 {
   nsresult result;
 
-  if (!aIterator)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aIterator, NS_ERROR_NULL_POINTER);
 
   nsCOMPtr<nsIDOMRange> range;
 
   result = CreateDocumentContentRange(getter_AddRefs(range));
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   result = CreateContentIterator(range, aIterator);
 
@@ -2276,13 +2241,11 @@ nsTextServicesDocument::AdjustContentIterator()
 {
   nsresult result = NS_OK;
 
-  if (!mIterator)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(mIterator, NS_ERROR_FAILURE);
 
   nsCOMPtr<nsIDOMNode> node(do_QueryInterface(mIterator->GetCurrentNode()));
 
-  if (!node)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(node, NS_ERROR_FAILURE);
 
   nsIDOMNode *nodePtr = node.get();
   PRInt32 tcount      = mOffsetTable.Length();
@@ -2296,8 +2259,7 @@ nsTextServicesDocument::AdjustContentIterator()
   {
     entry = mOffsetTable[i];
 
-    if (!entry)
-      return NS_ERROR_FAILURE;
+    NS_ENSURE_TRUE(entry, NS_ERROR_FAILURE);
 
     if (entry->mNode == nodePtr)
     {
@@ -2411,6 +2373,11 @@ nsTextServicesDocument::ClearDidSkip(nsIContentIterator* aFilteredIter)
 PRBool
 nsTextServicesDocument::IsBlockNode(nsIContent *aContent)
 {
+  if (!aContent) {
+    NS_ERROR("How did a null pointer get passed to IsBlockNode?");
+    return PR_FALSE;
+  }
+
   nsIAtom *atom = aContent->Tag();
 
   return (sAAtom       != atom &&
@@ -2471,8 +2438,7 @@ nsTextServicesDocument::HasSameBlockNodeParent(nsIContent *aContent1, nsIContent
 PRBool
 nsTextServicesDocument::IsTextNode(nsIContent *aContent)
 {
-  if (!aContent)
-    return PR_FALSE;
+  NS_ENSURE_TRUE(aContent, PR_FALSE);
 
   nsCOMPtr<nsIDOMNode> node = do_QueryInterface(aContent);
 
@@ -2482,15 +2448,13 @@ nsTextServicesDocument::IsTextNode(nsIContent *aContent)
 PRBool
 nsTextServicesDocument::IsTextNode(nsIDOMNode *aNode)
 {
-  if (!aNode)
-    return PR_FALSE;
+  NS_ENSURE_TRUE(aNode, PR_FALSE);
 
   PRUint16 type;
 
   nsresult result = aNode->GetNodeType(&type);
 
-  if (NS_FAILED(result))
-    return PR_FALSE;
+  NS_ENSURE_SUCCESS(result, PR_FALSE);
 
   return nsIDOMNode::TEXT_NODE == type;
 }
@@ -2500,8 +2464,7 @@ nsTextServicesDocument::SetSelectionInternal(PRInt32 aOffset, PRInt32 aLength, P
 {
   nsresult result = NS_OK;
 
-  if (!mSelCon || aOffset < 0 || aLength < 0)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(mSelCon && aOffset >= 0 && aLength >= 0, NS_ERROR_FAILURE);
 
   nsIDOMNode *sNode = 0, *eNode = 0;
   PRInt32 i, sOffset = 0, eOffset = 0;
@@ -2568,8 +2531,7 @@ nsTextServicesDocument::SetSelectionInternal(PRInt32 aOffset, PRInt32 aLength, P
     }
   }
 
-  if (!sNode)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(sNode, NS_ERROR_FAILURE);
 
   // XXX: If we ever get a SetSelection() method in nsIEditor, we should
   //      use it.
@@ -2580,13 +2542,11 @@ nsTextServicesDocument::SetSelectionInternal(PRInt32 aOffset, PRInt32 aLength, P
   {
     result = mSelCon->GetSelection(nsISelectionController::SELECTION_NORMAL, getter_AddRefs(selection));
 
-    if (NS_FAILED(result))
-      return result;
+    NS_ENSURE_SUCCESS(result, result);
 
     result = selection->Collapse(sNode, sOffset);
 
-    if (NS_FAILED(result))
-      return result;
+    NS_ENSURE_SUCCESS(result, result);
    }
 
   if (aLength <= 0)
@@ -2642,8 +2602,7 @@ nsTextServicesDocument::SetSelectionInternal(PRInt32 aOffset, PRInt32 aLength, P
   {
     result = selection->Extend(eNode, eOffset);
 
-    if (NS_FAILED(result))
-      return result;
+    NS_ENSURE_SUCCESS(result, result);
   }
 
   //**** KDEBUG ****
@@ -2658,15 +2617,13 @@ nsTextServicesDocument::GetSelection(nsITextServicesDocument::TSDBlockSelectionS
 {
   nsresult result;
 
-  if (!aSelStatus || !aSelOffset || !aSelLength)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aSelStatus && aSelOffset && aSelLength, NS_ERROR_NULL_POINTER);
 
   *aSelStatus = nsITextServicesDocument::eBlockNotFound;
   *aSelOffset = -1;
   *aSelLength = -1;
 
-  if (!mDOMDocument || !mSelCon)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(mDOMDocument && mSelCon, NS_ERROR_FAILURE);
 
   if (mIteratorStatus == nsTextServicesDocument::eIsDone)
     return NS_OK;
@@ -2676,16 +2633,13 @@ nsTextServicesDocument::GetSelection(nsITextServicesDocument::TSDBlockSelectionS
 
   result = mSelCon->GetSelection(nsISelectionController::SELECTION_NORMAL, getter_AddRefs(selection));
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
-  if (!selection)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(selection, NS_ERROR_FAILURE);
 
   result = selection->GetIsCollapsed(&isCollapsed);
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   // XXX: If we expose this method publicly, we need to
   //      add LOCK_DOC/UNLOCK_DOC calls!
@@ -2710,11 +2664,9 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
 
   result = mSelCon->GetSelection(nsISelectionController::SELECTION_NORMAL, getter_AddRefs(selection));
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
-  if (!selection)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(selection, NS_ERROR_FAILURE);
 
   // The calling function should have done the GetIsCollapsed()
   // check already. Just assume it's collapsed!
@@ -2752,28 +2704,23 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
 
   result = selection->GetRangeAt(0, getter_AddRefs(range));
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   result = range->GetStartContainer(getter_AddRefs(parent));
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   result = range->GetStartOffset(&offset);
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   result = ComparePoints(eStart->mNode, eStartOffset, parent, offset, &e1s1);
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   result = ComparePoints(eEnd->mNode, eEndOffset, parent, offset, &e2s1);
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   if (e1s1 > 0 || e2s1 < 0)
   {
@@ -2793,8 +2740,7 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
     {
       entry = mOffsetTable[i];
 
-      if (!entry)
-        return NS_ERROR_FAILURE;
+      NS_ENSURE_TRUE(entry, NS_ERROR_FAILURE);
 
       if (entry->mNode == parent.get() &&
           entry->mNodeOffset <= offset && offset <= (entry->mNodeOffset + entry->mLength))
@@ -2827,18 +2773,15 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
 
   result = CreateRange(eStart->mNode, eStartOffset, eEnd->mNode, eEndOffset, getter_AddRefs(range));
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   result = CreateContentIterator(range, getter_AddRefs(iter));
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   result = parent->HasChildNodes(&hasChildren);
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   if (hasChildren)
   {
@@ -2852,11 +2795,9 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
 
     result = parent->GetChildNodes(getter_AddRefs(children));
 
-    if (NS_FAILED(result))
-      return result;
+    NS_ENSURE_SUCCESS(result, result);
 
-    if (!children)
-      return NS_ERROR_FAILURE;
+    NS_ENSURE_TRUE(children, NS_ERROR_FAILURE);
 
     if (childIndex > 0)
     {
@@ -2864,8 +2805,7 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
 
       result = children->GetLength(&numChildren);
 
-      if (NS_FAILED(result))
-        return result;
+      NS_ENSURE_SUCCESS(result, result);
 
       NS_ASSERTION(childIndex <= numChildren, "Invalid selection offset!");
 
@@ -2877,18 +2817,15 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
 
     result = children->Item(childIndex, getter_AddRefs(saveNode));
 
-    if (NS_FAILED(result))
-      return result;
+    NS_ENSURE_SUCCESS(result, result);
 
     nsCOMPtr<nsIContent> content(do_QueryInterface(saveNode));
 
-    if (!content)
-      return NS_ERROR_FAILURE;
+    NS_ENSURE_TRUE(content, NS_ERROR_FAILURE);
 
     result = iter->PositionAt(content);
 
-    if (NS_FAILED(result))
-      return result;
+    NS_ENSURE_SUCCESS(result, result);
   }
   else
   {
@@ -2897,13 +2834,11 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
 
     nsCOMPtr<nsIContent> content(do_QueryInterface(parent));
 
-    if (!content)
-      return NS_ERROR_FAILURE;
+    NS_ENSURE_TRUE(content, NS_ERROR_FAILURE);
 
     result = iter->PositionAt(content);
 
-    if (NS_FAILED(result))
-      return result;
+    NS_ENSURE_SUCCESS(result, result);
 
     saveNode = parent;
   }
@@ -2920,8 +2855,7 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
     {
       node = do_QueryInterface(content);
 
-      if (!node)
-        return NS_ERROR_FAILURE;
+      NS_ENSURE_TRUE(node, NS_ERROR_FAILURE);
 
       break;
     }
@@ -2939,8 +2873,7 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
     nsAutoString str;
     result = node->GetNodeValue(str);
 
-    if (NS_FAILED(result))
-      return result;
+    NS_ENSURE_SUCCESS(result, result);
 
     offset = str.Length();
   }
@@ -2957,8 +2890,7 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
 
       result = iter->PositionAt(content);
 
-      if (NS_FAILED(result))
-        return result;
+      NS_ENSURE_SUCCESS(result, result);
     }
 
     while (!iter->IsDone())
@@ -2969,8 +2901,7 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
       {
         node = do_QueryInterface(content);
 
-        if (!node)
-          return NS_ERROR_FAILURE;
+        NS_ENSURE_TRUE(node, NS_ERROR_FAILURE);
 
         break;
       }
@@ -2980,8 +2911,7 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
       iter->Next();
     }
 
-    if (!node)
-      return NS_ERROR_FAILURE;
+    NS_ENSURE_TRUE(node, NS_ERROR_FAILURE);
 
     // We found a text node, so set the offset to
     // the begining of the node.
@@ -2993,8 +2923,7 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
   {
     entry = mOffsetTable[i];
 
-    if (!entry)
-      return NS_ERROR_FAILURE;
+    NS_ENSURE_TRUE(entry, NS_ERROR_FAILURE);
 
     if (entry->mNode == node.get() &&
         entry->mNodeOffset <= offset && offset <= (entry->mNodeOffset + entry->mLength))
@@ -3029,11 +2958,9 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
 
   result = mSelCon->GetSelection(nsISelectionController::SELECTION_NORMAL, getter_AddRefs(selection));
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
-  if (!selection)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(selection, NS_ERROR_FAILURE);
 
   // It is assumed that the calling function has made sure that the
   // selection is not collapsed, and that the input params to this
@@ -3064,8 +2991,7 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
 
   result = selection->GetRangeCount(&rangeCount);
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   // Find the first range in the selection that intersects
   // the current text block.
@@ -3074,25 +3000,21 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
   {
     result = selection->GetRangeAt(i, getter_AddRefs(range));
 
-    if (NS_FAILED(result))
-      return result;
+    NS_ENSURE_SUCCESS(result, result);
 
     result = GetRangeEndPoints(range,
                                getter_AddRefs(startParent), &startOffset,
                                getter_AddRefs(endParent), &endOffset);
 
-    if (NS_FAILED(result))
-      return result;
+    NS_ENSURE_SUCCESS(result, result);
 
     result = ComparePoints(eStart->mNode, eStartOffset, endParent, endOffset, &e1s2);
 
-    if (NS_FAILED(result))
-      return result;
+    NS_ENSURE_SUCCESS(result, result);
 
     result = ComparePoints(eEnd->mNode, eEndOffset, startParent, startOffset, &e2s1);
 
-    if (NS_FAILED(result))
-      return result;
+    NS_ENSURE_SUCCESS(result, result);
 
     // Break out of the loop if the text block intersects the current range.
 
@@ -3113,13 +3035,11 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
 
   result = ComparePoints(eStart->mNode, eStartOffset, startParent, startOffset, &e1s1);
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   result = ComparePoints(eEnd->mNode, eEndOffset, endParent, endOffset, &e2s2);
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   if (rangeCount > 1)
   {
@@ -3186,8 +3106,7 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
 
   result = CreateRange(p1, o1, p2, o2, getter_AddRefs(range));
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   // Now iterate over this range to figure out the selection's
   // block offset and length.
@@ -3196,8 +3115,7 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
 
   result = CreateContentIterator(range, getter_AddRefs(iter));
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   // Find the first text node in the range.
   
@@ -3218,8 +3136,7 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
       {
         p1 = do_QueryInterface(content);
 
-        if (!p1)
-          return NS_ERROR_FAILURE;
+        NS_ENSURE_TRUE(p1, NS_ERROR_FAILURE);
 
         o1 = 0;
         found = PR_TRUE;
@@ -3230,8 +3147,7 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
       iter->Next();
     }
 
-    if (!found)
-      return NS_ERROR_FAILURE;
+    NS_ENSURE_TRUE(found, NS_ERROR_FAILURE);
   }
 
   // Find the last text node in the range.
@@ -3250,15 +3166,13 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
       {
         p2 = do_QueryInterface(content);
 
-        if (!p2)
-          return NS_ERROR_FAILURE;
+        NS_ENSURE_TRUE(p2, NS_ERROR_FAILURE);
 
         nsString str;
 
         result = p2->GetNodeValue(str);
 
-        if (NS_FAILED(result))
-          return result;
+        NS_ENSURE_SUCCESS(result, result);
 
         o2 = str.Length();
         found = PR_TRUE;
@@ -3269,8 +3183,7 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
       iter->Prev();
     }
 
-    if (!found)
-      return NS_ERROR_FAILURE;
+    NS_ENSURE_TRUE(found, NS_ERROR_FAILURE);
   }
 
   found    = PR_FALSE;
@@ -3280,8 +3193,7 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
   {
     entry = mOffsetTable[i];
 
-    if (!entry)
-      return NS_ERROR_FAILURE;
+    NS_ENSURE_TRUE(entry, NS_ERROR_FAILURE);
 
     if (!found)
     {
@@ -3355,8 +3267,7 @@ nsTextServicesDocument::ComparePoints(nsIDOMNode* aParent1, PRInt32 aOffset1,
   if (!sRangeHelper) {
     result = CallGetService("@mozilla.org/content/range-utils;1",
                             &sRangeHelper);
-    if (!sRangeHelper)
-      return result;
+    NS_ENSURE_TRUE(sRangeHelper, result);
   }
 
   *aResult = sRangeHelper->ComparePoints(aParent1, aOffset1,
@@ -3371,29 +3282,23 @@ nsTextServicesDocument::GetRangeEndPoints(nsIDOMRange *aRange,
 {
   nsresult result;
 
-  if (!aRange || !aStartParent || !aStartOffset || !aEndParent || !aEndOffset)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aRange && aStartParent && aStartOffset && aEndParent && aEndOffset, NS_ERROR_NULL_POINTER);
 
   result = aRange->GetStartContainer(aStartParent);
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
-  if (!aStartParent)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(aStartParent, NS_ERROR_FAILURE);
 
   result = aRange->GetStartOffset(aStartOffset);
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   result = aRange->GetEndContainer(aEndParent);
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
-  if (!aEndParent)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(aEndParent, NS_ERROR_FAILURE);
 
   result = aRange->GetEndOffset(aEndOffset);
 
@@ -3409,11 +3314,9 @@ nsTextServicesDocument::CreateRange(nsIDOMNode *aStartParent, PRInt32 aStartOffs
   nsresult result;
 
   result = CallCreateInstance("@mozilla.org/content/range;1", aRange);
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
-  if (!*aRange)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(*aRange, NS_ERROR_NULL_POINTER);
 
   result = (*aRange)->SetStart(aStartParent, aStartOffset);
 
@@ -3484,8 +3387,7 @@ nsTextServicesDocument::LastTextNode(nsIContentIterator *aIterator,
 nsresult
 nsTextServicesDocument::FirstTextNodeInCurrentBlock(nsIContentIterator *iter)
 {
-  if (!iter)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(iter, NS_ERROR_NULL_POINTER);
 
   ClearDidSkip(iter);
 
@@ -3532,8 +3434,7 @@ nsTextServicesDocument::FirstTextNodeInPrevBlock(nsIContentIterator *aIterator)
   nsCOMPtr<nsIContent> content;
   nsresult result;
 
-  if (!aIterator)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aIterator, NS_ERROR_NULL_POINTER);
 
   // XXX: What if mIterator is not currently on a text node?
 
@@ -3542,8 +3443,7 @@ nsTextServicesDocument::FirstTextNodeInPrevBlock(nsIContentIterator *aIterator)
 
   result = FirstTextNodeInCurrentBlock(aIterator);
 
-  if (NS_FAILED(result))
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_SUCCESS(result, NS_ERROR_FAILURE);
 
   // Point mIterator to the first node before the first text node:
 
@@ -3563,8 +3463,7 @@ nsTextServicesDocument::FirstTextNodeInNextBlock(nsIContentIterator *aIterator)
   nsCOMPtr<nsIContent> prev;
   PRBool crossedBlockBoundary = PR_FALSE;
 
-  if (!aIterator)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aIterator, NS_ERROR_NULL_POINTER);
 
   ClearDidSkip(aIterator);
 
@@ -3596,8 +3495,7 @@ nsTextServicesDocument::GetFirstTextNodeInPrevBlock(nsIContent **aContent)
 {
   nsresult result;
 
-  if (!aContent)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aContent, NS_ERROR_NULL_POINTER);
 
   *aContent = 0;
 
@@ -3631,8 +3529,7 @@ nsTextServicesDocument::GetFirstTextNodeInNextBlock(nsIContent **aContent)
 {
   nsresult result;
 
-  if (!aContent)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aContent, NS_ERROR_NULL_POINTER);
 
   *aContent = 0;
 
@@ -3672,8 +3569,7 @@ nsTextServicesDocument::CreateOffsetTable(nsTArray<OffsetEntry*> *aOffsetTable,
   nsCOMPtr<nsIContent> first;
   nsCOMPtr<nsIContent> prev;
 
-  if (!aIterator)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aIterator, NS_ERROR_NULL_POINTER);
 
   ClearOffsetTable(aOffsetTable);
 
@@ -3705,8 +3601,7 @@ nsTextServicesDocument::CreateOffsetTable(nsTArray<OffsetEntry*> *aOffsetTable,
 
   result = FirstTextNodeInCurrentBlock(aIterator);
 
-  if (NS_FAILED(result))
-    return result;
+  NS_ENSURE_SUCCESS(result, result);
 
   PRInt32 offset = 0;
 
@@ -3728,15 +3623,13 @@ nsTextServicesDocument::CreateOffsetTable(nsTArray<OffsetEntry*> *aOffsetTable,
 
           result = node->GetNodeValue(str);
 
-          if (NS_FAILED(result))
-            return result;
+          NS_ENSURE_SUCCESS(result, result);
 
           // Add an entry for this text node into the offset table:
 
           OffsetEntry *entry = new OffsetEntry(node, offset, str.Length());
 
-          if (!entry)
-            return NS_ERROR_OUT_OF_MEMORY;
+          NS_ENSURE_TRUE(entry, NS_ERROR_OUT_OF_MEMORY);
 
           aOffsetTable->AppendElement(entry);
 
@@ -3882,8 +3775,7 @@ nsTextServicesDocument::SplitOffsetEntry(PRInt32 aTableIndex, PRInt32 aNewEntryL
                                           entry->mStrOffset + oldLength,
                                           aNewEntryLength);
 
-  if (!newEntry)
-    return NS_ERROR_OUT_OF_MEMORY;
+  NS_ENSURE_TRUE(newEntry, NS_ERROR_OUT_OF_MEMORY);
 
   if (!mOffsetTable.InsertElementAt(aTableIndex + 1, newEntry))
   {
@@ -3905,15 +3797,13 @@ nsTextServicesDocument::NodeHasOffsetEntry(nsTArray<OffsetEntry*> *aOffsetTable,
   OffsetEntry *entry;
   PRUint32 i;
 
-  if (!aNode || !aHasEntry || !aEntryIndex)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(aNode && aHasEntry && aEntryIndex, NS_ERROR_NULL_POINTER);
 
   for (i = 0; i < aOffsetTable->Length(); i++)
   {
     entry = (*aOffsetTable)[i];
 
-    if (!entry)
-      return NS_ERROR_FAILURE;
+    NS_ENSURE_TRUE(entry, NS_ERROR_FAILURE);
 
     if (entry->mNode == aNode)
     {
@@ -3958,8 +3848,8 @@ nsTextServicesDocument::FindWordBounds(nsTArray<OffsetEntry*> *aOffsetTable,
   if (aWordEndOffset)
     *aWordEndOffset = 0;
 
-  PRInt32 entryIndex;
-  PRBool hasEntry;
+  PRInt32 entryIndex = 0;
+  PRBool hasEntry = PR_FALSE;
 
   // It's assumed that aNode is a text node. The first thing
   // we do is get it's index in the offset table so we can
@@ -4024,8 +3914,9 @@ nsTextServicesDocument::FindWordBounds(nsTArray<OffsetEntry*> *aOffsetTable,
     // character covered by this entry, we will use the next
     // entry if there is one.
 
-    if (entry->mStrOffset <= res.mBegin &&
-       (res.mBegin < strEndOffset || (res.mBegin == strEndOffset && i == lastIndex)))
+    if (PRUint32(entry->mStrOffset) <= res.mBegin &&
+        (res.mBegin < PRUint32(strEndOffset) ||
+        (res.mBegin == PRUint32(strEndOffset) && i == lastIndex)))
     {
       if (aWordStartNode)
       {
@@ -4048,9 +3939,9 @@ nsTextServicesDocument::FindWordBounds(nsTArray<OffsetEntry*> *aOffsetTable,
     // Check to see if res.mEnd is within the range covered
     // by this entry.
 
-    if (entry->mStrOffset <= res.mEnd && res.mEnd <= strEndOffset)
+    if (PRUint32(entry->mStrOffset) <= res.mEnd && res.mEnd <= PRUint32(strEndOffset))
     {
-      if (res.mBegin == res.mEnd && res.mEnd == strEndOffset && i != lastIndex)
+      if (res.mBegin == res.mEnd && res.mEnd == PRUint32(strEndOffset) && i != lastIndex)
       {
         // Wait for the next round so that we use the same entry
         // we did for aWordStartNode.
