@@ -123,7 +123,7 @@ ArrayBuffer::class_finalize(JSContext *cx, JSObject *obj)
     ArrayBuffer *abuf = ArrayBuffer::fromJSObject(obj);
     if (abuf) {
         abuf->freeStorage(cx);
-        cx->destroy<ArrayBuffer>(abuf);
+        cx->delete_(abuf);
     }
 }
 
@@ -161,12 +161,12 @@ ArrayBuffer::create(JSContext *cx, int32 nbytes)
         return NULL;
     }
 
-    ArrayBuffer *abuf = cx->create<ArrayBuffer>();
+    ArrayBuffer *abuf = cx->new_<ArrayBuffer>();
     if (!abuf)
         return NULL;
 
     if (!abuf->allocateStorage(cx, nbytes)) {
-        cx->destroy<ArrayBuffer>(abuf);
+        Foreground::delete_(abuf);
         return NULL;
     }
 
@@ -180,7 +180,7 @@ ArrayBuffer::allocateStorage(JSContext *cx, uint32 nbytes)
     JS_ASSERT(data == 0);
 
     if (nbytes) {
-        data = cx->calloc(nbytes);
+        data = cx->calloc_(nbytes);
         if (!data)
             return false;
     }
@@ -193,7 +193,7 @@ void
 ArrayBuffer::freeStorage(JSContext *cx)
 {
     if (data) {
-        cx->free(data);
+        cx->free_(data);
 #ifdef DEBUG
         // the destructor asserts that data is 0 in debug builds
         data = NULL;
@@ -437,7 +437,7 @@ struct uint8_clamped {
         return *this;
     }
 
-    inline uint8_clamped& operator= (int32 x) { 
+    inline uint8_clamped& operator= (int32 x) {
         val = (x >= 0)
               ? ((x < 255)
                  ? uint8(x)
@@ -446,7 +446,7 @@ struct uint8_clamped {
         return *this;
     }
 
-    inline uint8_clamped& operator= (const jsdouble x) { 
+    inline uint8_clamped& operator= (const jsdouble x) {
         val = uint8(js_TypedArray_uint8_clamp_double(x));
         return *this;
     }
@@ -681,13 +681,13 @@ class TypedArrayTemplate
           case JSENUMERATE_INIT_ALL:
             statep->setBoolean(true);
             if (idp)
-                *idp = INT_TO_JSID(tarray->length + 1);
+                *idp = ::INT_TO_JSID(tarray->length + 1);
             break;
 
           case JSENUMERATE_INIT:
             statep->setInt32(0);
             if (idp)
-                *idp = INT_TO_JSID(tarray->length);
+                *idp = ::INT_TO_JSID(tarray->length);
             break;
 
           case JSENUMERATE_NEXT:
@@ -697,7 +697,7 @@ class TypedArrayTemplate
             } else {
                 uint32 index = statep->toInt32();
                 if (index < uint32(tarray->length)) {
-                    *idp = INT_TO_JSID(index);
+                    *idp = ::INT_TO_JSID(index);
                     statep->setInt32(index + 1);
                 } else {
                     JS_ASSERT(index == tarray->length);
@@ -727,7 +727,7 @@ class TypedArrayTemplate
         if (!obj)
             return NULL;
 
-        ThisTypeArray *tarray = cx->create<ThisTypeArray>(bufobj, byteOffset, len);
+        ThisTypeArray *tarray = cx->new_<ThisTypeArray>(bufobj, byteOffset, len);
         if (!tarray)
             return NULL;
 
@@ -832,7 +832,7 @@ class TypedArrayTemplate
     {
         ThisTypeArray *tarray = ThisTypeArray::fromJSObject(obj);
         if (tarray)
-            cx->destroy<ThisTypeArray>(tarray);
+            cx->delete_(tarray);
     }
 
     /* subarray(start[, end]) */
@@ -843,14 +843,9 @@ class TypedArrayTemplate
         if (!obj)
             return false;
 
-        if (!InstanceOf(cx, obj, ThisTypeArray::fastClass(), vp + 2))
-            return false;
-
         if (obj->getClass() != fastClass()) {
             // someone tried to apply this subarray() to the wrong class
-            JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL,
-                                 JSMSG_INCOMPATIBLE_METHOD,
-                                 fastClass()->name, "subarray", obj->getClass()->name);
+            ReportIncompatibleMethod(cx, vp, fastClass());
             return false;
         }
 
@@ -905,14 +900,9 @@ class TypedArrayTemplate
         if (!obj)
             return false;
 
-        if (!InstanceOf(cx, obj, ThisTypeArray::fastClass(), vp + 2))
-            return false;
-
         if (obj->getClass() != fastClass()) {
             // someone tried to apply this set() to the wrong class
-            JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL,
-                                 JSMSG_INCOMPATIBLE_METHOD,
-                                 fastClass()->name, "set", obj->getClass()->name);
+            ReportIncompatibleMethod(cx, vp, fastClass());
             return false;
         }
 
@@ -1140,7 +1130,7 @@ class TypedArrayTemplate
 
         return NativeType(int32(0));
     }
-    
+
     static bool
     copyFrom(JSContext *cx, JSObject *thisTypedArrayObj,
              JSObject *ar, jsuint len, jsuint offset = 0)
@@ -1164,7 +1154,7 @@ class TypedArrayTemplate
             Value v;
 
             for (uintN i = 0; i < len; ++i) {
-                if (!ar->getProperty(cx, INT_TO_JSID(i), &v))
+                if (!ar->getProperty(cx, ::INT_TO_JSID(i), &v))
                     return false;
                 *dest++ = nativeFromValue(cx, v);
             }
@@ -1264,7 +1254,7 @@ class TypedArrayTemplate
 
         // We have to make a copy of the source array here, since
         // there's overlap, and we have to convert types.
-        void *srcbuf = cx->malloc(tarray->byteLength);
+        void *srcbuf = cx->malloc_(tarray->byteLength);
         if (!srcbuf)
             return false;
         memcpy(srcbuf, tarray->data, tarray->byteLength);
@@ -1324,7 +1314,7 @@ class TypedArrayTemplate
             break;
         }
 
-        js_free(srcbuf);
+        UnwantedForeground::free_(srcbuf);
         return true;
     }
 
@@ -1504,7 +1494,7 @@ template<> JSFunctionSpec _typedArray::jsfuncs[] = {                           \
     NULL,           /* construct   */                                          \
     NULL,           /* xdrObject   */                                          \
     NULL,           /* hasInstance */                                          \
-    NULL,           /* mark        */                                          \
+    _typedArray::obj_trace,                                                    \
     JS_NULL_CLASS_EXT,                                                         \
     {                                                                          \
         _typedArray::obj_lookupProperty,                                       \
@@ -1516,7 +1506,6 @@ template<> JSFunctionSpec _typedArray::jsfuncs[] = {                           \
         _typedArray::obj_deleteProperty,                                       \
         _typedArray::obj_enumerate,                                            \
         _typedArray::obj_typeOf,                                               \
-        _typedArray::obj_trace,                                                \
         NULL,       /* thisObject      */                                      \
         NULL,       /* clear           */                                      \
     }                                                                          \
