@@ -20,7 +20,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Igor Bukanov
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -37,22 +37,26 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-var summary = "Do not crash in js_IsXMLName";
-var BUGNUMBER = 327691;
-var actual = 'No Crash';
-var expect = 'No Crash';
+var BUGNUMBER = 356238;
+var summary = 'bug 356238';
+var actual = 'No Duplicate';
+var expect = 'No Duplicate';
 
 printBugNumber(BUGNUMBER);
 START(summary);
 
-var A=<y/>;
-var B=A.h;
-var D=B.h;
-B.h=D.h;
-B[0]=B.h;
+var xml = <xml><child><a/><b/></child></xml>;
+var child = xml.child[0];
 
-B.h=D.h; // Crashes at js_IsXMLName.
+try {
+    child.insertChildAfter(null, xml.child);
+    actual = "insertChildAfter succeeded when it should throw an exception";
+}
+catch (e) {
+}
+
+if (child.a[0] === child.a[1])
+    actual = 'Duplicate detected';
 
 TEST(1, expect, actual);
-
 END();
