@@ -38,46 +38,73 @@
 
 
 /**
-   File Name:          15.4.1.3.js
-   ECMA Section:       15.4.1.3 Array()
+   File Name:          15.4.1.1.js
+   ECMA Section:       15.4.1 Array( item0, item1,... )
 
    Description:        When Array is called as a function rather than as a
    constructor, it creates and initializes a new array
    object.  Thus, the function call Array(...) is
-   equivalent to the object creationi new Array(...) with
+   equivalent to the object creation new Array(...) with
    the same arguments.
 
-   An array is created and returned as if by the
-   expression new Array(len).
+   An array is created and returned as if by the expression
+   new Array( item0, item1, ... ).
 
    Author:             christine@netscape.com
    Date:               7 october 1997
 */
-var SECTION = "15.4.1.3";
+var SECTION = "15.4.1.1";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Array Constructor Called as a Function:  Array()";
+var TITLE   = "Array Constructor Called as a Function";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-new TestCase(   SECTION,
-		"typeof Array()",
-		"object",
-		typeof Array() );
+new TestCase( SECTION,
+	      "typeof Array(1,2)",       
+	      "object",          
+	      typeof Array(1,2) );
 
-new TestCase(   SECTION,
-		"MYARR = new Array();MYARR.getClass = Object.prototype.toString;MYARR.getClass()",
-		"[object Array]",
-		eval("MYARR = Array();MYARR.getClass = Object.prototype.toString;MYARR.getClass()") );
+new TestCase( SECTION,
+	      "(Array(1,2)).toString",   
+	      Array.prototype.toString,   
+	      (Array(1,2)).toString );
 
-new TestCase(   SECTION,
-		"(Array()).length",
-		0,         
-		(Array()).length );
+new TestCase( SECTION,
+	      "var arr = Array(1,2,3); arr.toString = Object.prototype.toString; arr.toString()",
+	      "[object Array]",
+	      eval("var arr = Array(1,2,3); arr.toString = Object.prototype.toString; arr.toString()") );
 
-new TestCase(   SECTION,
-		"Array().toString()",
-		"",
-		Array().toString() );
+new TestCase( SECTION,
+	      "(Array(1,2)).length",     
+	      2,                 
+	      (Array(1,2)).length );
+
+new TestCase( SECTION,
+	      "var arr = (Array(1,2)); arr[0]", 
+	      1,          
+	      eval("var arr = (Array(1,2)); arr[0]") );
+
+new TestCase( SECTION,
+	      "var arr = (Array(1,2)); arr[1]", 
+	      2,          
+	      eval("var arr = (Array(1,2)); arr[1]") );
+
+new TestCase( SECTION,
+	      "var arr = (Array(1,2)); String(arr)", 
+	      "1,2", 
+	      eval("var arr = (Array(1,2)); String(arr)") );
 
 test();
+
+function ToUint32( n ) {
+  n = Number( n );
+  if( isNaN(n) || n == 0 || n == Number.POSITIVE_INFINITY ||
+      n == Number.NEGATIVE_INFINITY ) {
+    return 0;
+  }
+  var sign = n < 0 ? -1 : 1;
+
+  return ( sign * ( n * Math.floor( Math.abs(n) ) ) ) % Math.pow(2, 32);
+}
+
