@@ -38,57 +38,75 @@
 
 
 /**
-   File Name:          15.9.1.1-1.js
-   ECMA Section:       15.9.1.1 Time Range
-   Description:
-   - leap seconds are ignored
-   - assume 86400000 ms / day
-   - numbers range fom +/- 9,007,199,254,740,991
-   - ms precision for any instant that is within
-   approximately +/-285,616 years from 1 jan 1970
-   UTC
-   - range of times supported is -100,000,000 days
-   to 100,000,000 days from 1 jan 1970 12:00 am
-   - time supported is 8.64e5*10e8 milliseconds from
-   1 jan 1970 UTC  (+/-273972.6027397 years)
+   File Name:          15.9.5.23-2.js
+   ECMA Section:       15.9.5.23
+   Description:        Date.prototype.setTime
+
+   1.  If the this value is not a Date object, generate a runtime error.
+   2.  Call ToNumber(time).
+   3.  Call TimeClip(Result(1)).
+   4.  Set the [[Value]] property of the this value to Result(2).
+   5.  Return the value of the [[Value]] property of the this value.
 
-   -   this test generates its own data -- it does not
-   read data from a file.
    Author:             christine@netscape.com
-   Date:               7 july 1997
-
-   Static variables:
-   FOUR_HUNDRED_YEARS
-
+   Date:               12 november 1997
 */
 
-//  every one hundred years contains:
-//    24 years with 366 days
-//
-//  every four hundred years contains:
-//    97 years with 366 days
-//   303 years with 365 days
-//
-//   86400000*365*97    =    3067372800000
-//  +86400000*366*303   =  + 9555408000000
-//                      =    1.26227808e+13
-var FOUR_HUNDRED_YEARS = 1.26227808e+13;
-var SECTION         =  "15.9.1.1-1";
+var SECTION = "15.9.5.23-2";
+var VERSION = "ECMA_1";
+startTest();
+var TITLE   = "Date.prototype.setTime()";
 
-writeHeaderToLog("15.9.1.1 Time Range");
+writeHeaderToLog( SECTION + " "+ TITLE);
 
-var M_SECS;
-var CURRENT_YEAR;
+test_times = new Array( TIME_NOW, TIME_0000, TIME_1970, TIME_1900, TIME_2000,
+			UTC_FEB_29_2000, UTC_JAN_1_2005 );
 
-for ( M_SECS = 0, CURRENT_YEAR = 1970;
-      M_SECS < 8640000000000000;
-      M_SECS += FOUR_HUNDRED_YEARS, CURRENT_YEAR += 400 ) {
 
-  new TestCase( SECTION,  
-		"new Date("+M_SECS+")",
-		CURRENT_YEAR,
-		(new Date( M_SECS)).getUTCFullYear() );
+for ( var j = 0; j < test_times.length; j++ ) {
+  addTestCase( new Date(TIME_1970), test_times[j] );
 }
 
+
+new TestCase( SECTION,
+	      "(new Date(NaN)).setTime()",
+	      NaN,
+	      (new Date(NaN)).setTime() );
+
+new TestCase( SECTION,
+	      "Date.prototype.setTime.length",
+	      1,
+	      Date.prototype.setTime.length );
 test();
 
+function addTestCase( d, t ) {
+  new TestCase( SECTION,
+		"( "+d+" ).setTime("+t+")",
+		t,
+		d.setTime(t) );
+
+  new TestCase( SECTION,
+		"( "+d+" ).setTime("+(t+1.1)+")",
+		TimeClip(t+1.1),
+		d.setTime(t+1.1) );
+
+  new TestCase( SECTION,
+		"( "+d+" ).setTime("+(t+1)+")",
+		t+1,
+		d.setTime(t+1) );
+
+  new TestCase( SECTION,
+		"( "+d+" ).setTime("+(t-1)+")",
+		t-1,
+		d.setTime(t-1) );
+
+  new TestCase( SECTION,
+		"( "+d+" ).setTime("+(t-TZ_ADJUST)+")",
+		t-TZ_ADJUST,
+		d.setTime(t-TZ_ADJUST) );
+
+  new TestCase( SECTION,
+		"( "+d+" ).setTime("+(t+TZ_ADJUST)+")",
+		t+TZ_ADJUST,
+		d.setTime(t+TZ_ADJUST) );
+}
