@@ -38,26 +38,33 @@
 
 
 /**
-   File Name:          15.7.3.3-3.js
-   ECMA Section:       15.7.3.3 Number.MIN_VALUE
+   File Name:          15.7.3.1-2.js
+   ECMA Section:       15.7.3.1 Number.prototype
    Description:        All value properties of the Number object should have
    the attributes [DontEnum, DontDelete, ReadOnly]
 
-   this test checks the ReadOnly attribute of Number.MIN_VALUE
+   this test checks the ReadOnly attribute of Number.prototype
 
    Author:             christine@netscape.com
    Date:               16 september 1997
 */
-var SECTION = "15.7.3.3-3";
+
+
+var SECTION = "15.7.3.1-2";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Number.MIN_VALUE:  ReadOnly Attribute";
+var TITLE   = "Number.prototype";
 
-writeHeaderToLog( SECTION + " "+TITLE );
+writeHeaderToLog( SECTION + " "+ TITLE);
 
-new TestCase( SECTION,
-	      "Number.MIN_VALUE=0; Number.MIN_VALUE",
-	      Number.MIN_VALUE,
-	      eval("Number.MIN_VALUE=0; Number.MIN_VALUE" ));
+new TestCase(   SECTION,
+		"var NUM_PROT = Number.prototype; Number.prototype = null; Number.prototype == NUM_PROT",
+		true,
+		eval("var NUM_PROT = Number.prototype; Number.prototype = null; Number.prototype == NUM_PROT") );
+
+new TestCase(   SECTION,
+		"Number.prototype=0; Number.prototype",
+		Number.prototype,
+		eval("Number.prototype=0; Number.prototype") );
 
 test();
