@@ -16,11 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Reto Laemmler
- *                 Brendan Eich
+ * Contributor(s): Gary Kwong
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -37,52 +36,22 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 354541;
-var summary = 'Regression to standard class constructors in case labels';
+var BUGNUMBER = 469405;
+var summary = 'Do not assert: !JSVAL_IS_PRIMITIVE(regs.sp[-2])';
 var actual = '';
 var expect = '';
 
-
 printBugNumber(BUGNUMBER);
-printStatus (summary + ': top level');
+printStatus (summary);
 
-String.prototype.trim = function() { print('hallo'); };
-
-String.prototype.trim = function() { return 'hallo'; };
-
-const S = String;
-const Sp = String.prototype;
-
-expect = 'hallo';
-var expectStringInvariant = true
-  var actualStringInvariant;
-var expectStringPrototypeInvariant = true;
-var actualStringPrototypeInvariant;
-
-if (typeof Script == 'undefined')
-{
-  print('Test skipped. Script not defined.');
-  reportCompare("Script not defined, Test skipped.",
-                "Script not defined, Test skipped.",
-                summary);
+try
+{ 
+  eval("__proto__.__iterator__ = [].toString");
+  for (var z = 0; z < 3; ++z) { if (z % 3 == 2) { for(let y in []); } }
 }
-else
+catch(ex)
 {
-  var s = Script('var tmp = function(o) { switch(o) { case String: case 1: return ""; } }; actualStringInvariant = (String === S); actualStringPrototypeInvariant = (String.prototype === Sp); actual = "".trim();');
-  try
-  {
-    s();
-  }
-  catch(ex)
-  {
-    actual = ex + '';
-  }
-  reportCompare(expect, actual, 'trim() returned');
-  reportCompare(expectStringInvariant, actualStringInvariant,
-                'String invariant');
-  reportCompare(expectStringPrototypeInvariant,
-                actualStringPrototypeInvariant,
-                'String.prototype invariant');
-
+  print('caught ' + ex);
 }
- 
+
+reportCompare(expect, actual, summary);
