@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s):
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,8 +36,8 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 467495;
-var summary = 'TCF_FUN_CLOSURE_VS_VAR is necessary';
+var BUGNUMBER = 452498;
+var summary = 'TM: upvar2 regression tests';
 var actual = '';
 var expect = '';
 
@@ -52,23 +52,13 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-  function f(x)
-  {
-    var y = 1;
-    if (Math)
-      function x() { }
-    if (Math)
-      function y() { }
-    return [x, y];
-  }
+  expect = '1';
 
-  var r = f(0);
+// ------- Comment #77 From Brendan Eich
 
-  if (typeof(r[0]) != "function")
-    actual += "Bad r[0]";
+  (function () { const [d] = [1]; [d] = [2]; print(actual = d);})();
 
-  if (typeof(r[1]) != "function")
-    throw "Bad r[1]";
+  actual = String(actual);
 
   reportCompare(expect, actual, summary);
 
