@@ -60,7 +60,7 @@ typedef RenderFrameParent::ViewMap ViewMap;
 
 // Represents (affine) transforms that are calculated from a content view.
 struct ViewTransform {
-  ViewTransform(nsIntPoint aTranslation, float aXScale, float aYScale)
+  ViewTransform(nsIntPoint aTranslation = nsIntPoint(0, 0), float aXScale = 1, float aYScale = 1)
     : mTranslation(aTranslation)
     , mXScale(aXScale)
     , mYScale(aYScale)
@@ -100,6 +100,13 @@ static void Scale(gfx3DMatrix& aTransform, double aXScale, double aYScale)
   aTransform._22 *= aYScale;
 }
 
+static void ReverseTranslate(gfx3DMatrix& aTransform, ViewTransform& aViewTransform)
+{
+  aTransform._41 -= aViewTransform.mTranslation.x / aViewTransform.mXScale;
+  aTransform._42 -= aViewTransform.mTranslation.y / aViewTransform.mYScale;
+}
+
+
 static void ApplyTransform(nsRect& aRect,
                            gfx3DMatrix& aTransform,
                            nscoord auPerDevPixel)
@@ -132,13 +139,8 @@ FindViewForId(const ViewMap& aMap, ViewID aId)
 static const FrameMetrics*
 GetFrameMetrics(Layer* aLayer)
 {
-  // Children are not container layers, so they don't have frame metrics. Give
-  // them a blank metric.
-  if (!aLayer->GetFirstChild())
-    return NULL;
-
-  ContainerLayer* container = static_cast<ContainerLayer*>(aLayer);
-  return &container->GetFrameMetrics();
+  ContainerLayer* container = aLayer->AsContainerLayer();
+  return container ? &container->GetFrameMetrics() : NULL;
 }
 
 static nsIntPoint
@@ -181,7 +183,11 @@ ComputeShadowTreeTransform(nsIFrame* aContainerFrame,
     aConfig.mScrollOffset.ToNearestPixels(auPerDevPixel);
   nsIntPoint metricsScrollOffset = aMetrics->mViewportScrollOffset;
 
-  if (aRootFrameLoader->AsyncScrollEnabled()) {
+  if (aRootFrameLoader->AsyncScrollEnabled() && !aMetrics->mDisplayPort.IsEmpty()) {
+    // Only use asynchronous scrolling if it is enabled and there is a
+    // displayport defined. It is useful to have a scroll layer that is
+    // synchronously scrolled for identifying a scroll area before it is
+    // being actively scrolled.
     nsIntPoint scrollCompensation(
       scrollOffset.x * aInverseScaleX - metricsScrollOffset.x * aConfig.mXScale,
       scrollOffset.y * aInverseScaleY - metricsScrollOffset.y * aConfig.mYScale);
@@ -252,7 +258,7 @@ BuildListForLayer(Layer* aLayer,
 static void
 TransformShadowTree(nsDisplayListBuilder* aBuilder, nsFrameLoader* aFrameLoader,
                     nsIFrame* aFrame, Layer* aLayer,
-                    float aXScale = 1, float aYScale = 1)
+                    ViewTransform& aTransform)
 {
   ShadowLayer* shadow = aLayer->AsShadowLayer();
   shadow->SetShadowClipRect(aLayer->GetClipRect());
@@ -270,10 +276,11 @@ TransformShadowTree(nsDisplayListBuilder* aBuilder, nsFrameLoader* aFrameLoader,
 
     ViewTransform viewTransform = ComputeShadowTreeTransform(
       aFrame, aFrameLoader, metrics, view->GetViewConfig(),
-      1 / aXScale, 1 / aYScale
+      1 / aTransform.mXScale, 1 / aTransform.mYScale
     );
 
     if (metrics->IsRootScrollable()) {
+      aTransform.mTranslation = viewTransform.mTranslation;
       viewTransform.mTranslation += GetRootFrameOffset(aFrame, aBuilder);
     }
 
@@ -283,13 +290,24 @@ TransformShadowTree(nsDisplayListBuilder* aBuilder, nsFrameLoader* aFrameLoader,
     shadowTransform = aLayer->GetTransform();
   }
 
+  if (aLayer->GetIsFixedPosition() &&
+      !aLayer->GetParent()->GetIsFixedPosition()) {
+    ReverseTranslate(shadowTransform, aTransform);
+    const nsIntRect* clipRect = shadow->GetShadowClipRect();
+    if (clipRect) {
+      nsIntRect transformedClipRect(*clipRect);
+      transformedClipRect.MoveBy(shadowTransform._41, shadowTransform._42);
+      shadow->SetShadowClipRect(&transformedClipRect);
+    }
+  }
+
   shadow->SetShadowTransform(shadowTransform);
-  aXScale *= GetXScale(shadowTransform);
-  aYScale *= GetYScale(shadowTransform);
+  aTransform.mXScale *= GetXScale(shadowTransform);
+  aTransform.mYScale *= GetYScale(shadowTransform);
 
   for (Layer* child = aLayer->GetFirstChild();
        child; child = child->GetNextSibling()) {
-    TransformShadowTree(aBuilder, aFrameLoader, aFrame, child, aXScale, aYScale);
+    TransformShadowTree(aBuilder, aFrameLoader, aFrame, child, aTransform);
   }
 }
 
@@ -323,10 +341,9 @@ BuildViewMap(ViewMap& oldContentViews, ViewMap& newContentViews,
              nsFrameLoader* aFrameLoader, Layer* aLayer,
              float aXScale = 1, float aYScale = 1)
 {
-  if (!aLayer->GetFirstChild())
+  ContainerLayer* container = aLayer->AsContainerLayer();
+  if (!container)
     return;
-
-  ContainerLayer* container = static_cast<ContainerLayer*>(aLayer);
   const FrameMetrics metrics = container->GetFrameMetrics();
   const ViewID scrollId = metrics.mScrollId;
 
@@ -639,7 +656,8 @@ RenderFrameParent::BuildLayer(nsDisplayListBuilder* aBuilder,
   mContainer->InsertAfter(shadowRoot, nsnull);
 
   AssertInTopLevelChromeDoc(mContainer, aFrame);
-  TransformShadowTree(aBuilder, mFrameLoader, aFrame, shadowRoot);
+  ViewTransform transform;
+  TransformShadowTree(aBuilder, mFrameLoader, aFrame, shadowRoot, transform);
   mContainer->SetClipRect(nsnull);
 
   if (mFrameLoader->AsyncScrollEnabled()) {
