@@ -54,12 +54,10 @@
 #include "nsMenuPopupFrame.h"
 #include "nsMenuBarFrame.h"
 #include "nsIView.h"
-#include "nsIWidget.h"
 #include "nsIDocument.h"
 #include "nsIDOMNSDocument.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMElement.h"
-#include "nsISupportsArray.h"
 #include "nsIDOMText.h"
 #include "nsILookAndFeel.h"
 #include "nsIComponentManager.h"
@@ -73,7 +71,6 @@
 #include "nsIDOMKeyEvent.h"
 #include "nsEventDispatcher.h"
 #include "nsIPrivateDOMEvent.h"
-#include "nsIScrollableView.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
 #include "nsUnicharUtils.h"
@@ -83,6 +80,9 @@
 #include "nsDisplayList.h"
 #include "nsIReflowCallback.h"
 #include "nsISound.h"
+#include "nsEventStateManager.h"
+#include "nsIDOMXULMenuListElement.h"
+#include "mozilla/Services.h"
 
 #define NS_MENU_POPUP_LIST_INDEX 0
 
@@ -100,6 +100,7 @@ nsString *nsMenuFrame::gControlText = nsnull;
 nsString *nsMenuFrame::gMetaText = nsnull;
 nsString *nsMenuFrame::gAltText = nsnull;
 nsString *nsMenuFrame::gModifierSeparator = nsnull;
+const PRInt32 kBlinkDelay = 67; // milliseconds
 
 // this class is used for dispatching menu activation events asynchronously.
 class nsMenuActivateEvent : public nsRunnable
@@ -148,45 +149,74 @@ public:
 
 private:
   nsCOMPtr<nsIContent> mMenu;
-  nsCOMPtr<nsPresContext> mPresContext;
+  nsRefPtr<nsPresContext> mPresContext;
   PRBool mIsActivate;
 };
 
+class nsMenuAttributeChangedEvent : public nsRunnable
+{
+public:
+  nsMenuAttributeChangedEvent(nsIFrame* aFrame, nsIAtom* aAttr)
+  : mFrame(aFrame), mAttr(aAttr)
+  {
+  }
+
+  NS_IMETHOD Run()
+  {
+    nsMenuFrame* frame = static_cast<nsMenuFrame*>(mFrame.GetFrame());
+    NS_ENSURE_STATE(frame);
+    if (mAttr == nsGkAtoms::checked) {
+      frame->UpdateMenuSpecialState(frame->PresContext());
+    } else if (mAttr == nsGkAtoms::acceltext) {
+      // someone reset the accelText attribute,
+      // so clear the bit that says *we* set it
+      frame->RemoveStateBits(NS_STATE_ACCELTEXT_IS_DERIVED);
+      frame->BuildAcceleratorText(PR_TRUE);
+    }
+    else if (mAttr == nsGkAtoms::key) {
+      frame->BuildAcceleratorText(PR_TRUE);
+    } else if (mAttr == nsGkAtoms::type || mAttr == nsGkAtoms::name) {
+      frame->UpdateMenuType(frame->PresContext());
+    }
+    return NS_OK;
+  }
+protected:
+  nsWeakFrame       mFrame;
+  nsCOMPtr<nsIAtom> mAttr;
+};
+
 //
-// NS_NewMenuFrame
+// NS_NewMenuFrame and NS_NewMenuItemFrame
 //
-// Wrapper for creating a new menu popup container
+// Wrappers for creating a new menu popup container
 //
 nsIFrame*
-NS_NewMenuFrame(nsIPresShell* aPresShell, nsStyleContext* aContext, PRUint32 aFlags)
+NS_NewMenuFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
 {
   nsMenuFrame* it = new (aPresShell) nsMenuFrame (aPresShell, aContext);
   
-  if ((it != nsnull) && aFlags)
+  if (it)
     it->SetIsMenu(PR_TRUE);
 
   return it;
 }
 
-NS_IMETHODIMP_(nsrefcnt) 
-nsMenuFrame::AddRef(void)
+nsIFrame*
+NS_NewMenuItemFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
 {
-  return NS_OK;
+  nsMenuFrame* it = new (aPresShell) nsMenuFrame (aPresShell, aContext);
+
+  if (it)
+    it->SetIsMenu(PR_FALSE);
+
+  return it;
 }
 
-NS_IMETHODIMP_(nsrefcnt)
-nsMenuFrame::Release(void)
-{
-    return NS_OK;
-}
+NS_IMPL_FRAMEARENA_HELPERS(nsMenuFrame)
 
-//
-// QueryInterface
-//
-NS_INTERFACE_MAP_BEGIN(nsMenuFrame)
-  NS_INTERFACE_MAP_ENTRY(nsIMenuFrame)
-  NS_INTERFACE_MAP_ENTRY(nsIScrollableViewProvider)
-NS_INTERFACE_MAP_END_INHERITING(nsBoxFrame)
+NS_QUERYFRAME_HEAD(nsMenuFrame)
+  NS_QUERYFRAME_ENTRY(nsIMenuFrame)
+NS_QUERYFRAME_TAIL_INHERITING(nsBoxFrame)
 
 //
 // nsMenuFrame cntr
@@ -195,19 +225,20 @@ nsMenuFrame::nsMenuFrame(nsIPresShell* aShell, nsStyleContext* aContext):
   nsBoxFrame(aShell, aContext),
     mIsMenu(PR_FALSE),
     mChecked(PR_FALSE),
+    mIgnoreAccelTextChange(PR_FALSE),
     mType(eMenuType_Normal),
     mMenuParent(nsnull),
-    mPopupFrame(nsnull)
+    mPopupFrame(nsnull),
+    mBlinkState(0)
 {
 
 } // cntr
 
-NS_IMETHODIMP
-nsMenuFrame::SetParent(const nsIFrame* aParent)
+void
+nsMenuFrame::SetParent(nsIFrame* aParent)
 {
   nsBoxFrame::SetParent(aParent);
-  InitMenuParent(const_cast<nsIFrame *>(aParent));
-  return NS_OK;
+  InitMenuParent(aParent);
 }
 
 void
@@ -273,13 +304,13 @@ nsMenuFrame::Init(nsIContent*      aContent,
 
   //load the display strings for the keyboard accelerators, but only once
   if (gRefCnt++ == 0) {
-    
-    nsCOMPtr<nsIStringBundleService> bundleService(do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv));
+    nsCOMPtr<nsIStringBundleService> bundleService =
+      mozilla::services::GetStringBundleService();
     nsCOMPtr<nsIStringBundle> bundle;
-    if (NS_SUCCEEDED(rv) && bundleService) {
+    if (bundleService) {
       rv = bundleService->CreateBundle( "chrome://global-platform/locale/platformKeys.properties",
                                         getter_AddRefs(bundle));
-    }    
+    }
     
     NS_ASSERTION(NS_SUCCEEDED(rv) && bundle, "chrome://global/locale/platformKeys.properties could not be loaded");
     nsXPIDLString shiftModifier;
@@ -305,7 +336,7 @@ nsMenuFrame::Init(nsIContent*      aContent,
     gModifierSeparator = new nsString(modifierSeparator);    
   }
 
-  BuildAcceleratorText();
+  BuildAcceleratorText(PR_FALSE);
   nsIReflowCallback* cb = new nsASyncMenuInitialization(this);
   NS_ENSURE_TRUE(cb, NS_ERROR_OUT_OF_MEMORY);
   PresContext()->PresShell()->PostReflowCallback(cb);
@@ -331,42 +362,35 @@ nsMenuFrame::~nsMenuFrame()
 
 // The following methods are all overridden to ensure that the menupopup frame
 // is placed in the appropriate list.
-nsIFrame*
-nsMenuFrame::GetFirstChild(nsIAtom* aListName) const
+nsFrameList
+nsMenuFrame::GetChildList(nsIAtom* aListName) const
 {
   if (nsGkAtoms::popupList == aListName) {
-    return mPopupFrame;
+    return nsFrameList(mPopupFrame, mPopupFrame);
   }
-  return nsBoxFrame::GetFirstChild(aListName);
+  return nsBoxFrame::GetChildList(aListName);
 }
 
-nsIFrame*
-nsMenuFrame::SetPopupFrame(nsIFrame* aChildList)
+void
+nsMenuFrame::SetPopupFrame(nsFrameList& aFrameList)
 {
-  // Check for a menupopup and move it to mPopupFrame
-  nsFrameList frames(aChildList);
-  nsIFrame* frame = frames.FirstChild();
-  while (frame) {
-    if (frame->GetType() == nsGkAtoms::menuPopupFrame) {
+  for (nsFrameList::Enumerator e(aFrameList); !e.AtEnd(); e.Next()) {
+    if (e.get()->GetType() == nsGkAtoms::menuPopupFrame) {
       // Remove this frame from the list and set it as mPopupFrame
-      frames.RemoveFrame(frame);
-      mPopupFrame = (nsMenuPopupFrame *)frame;
-      aChildList = frames.FirstChild();
+      mPopupFrame = (nsMenuPopupFrame *)e.get();
+      aFrameList.RemoveFrame(e.get());
       break;
     }
-    frame = frame->GetNextSibling();
   }
-
-  return aChildList;
 }
 
 NS_IMETHODIMP
 nsMenuFrame::SetInitialChildList(nsIAtom*        aListName,
-                                 nsIFrame*       aChildList)
+                                 nsFrameList&    aChildList)
 {
   NS_ASSERTION(!mPopupFrame, "already have a popup frame set");
   if (!aListName || aListName == nsGkAtoms::popupList)
-    aChildList = SetPopupFrame(aChildList);
+    SetPopupFrame(aChildList);
   return nsBoxFrame::SetInitialChildList(aListName, aChildList);
 }
 
@@ -380,7 +404,7 @@ nsMenuFrame::GetAdditionalChildListName(PRInt32 aIndex) const
 }
 
 void
-nsMenuFrame::Destroy()
+nsMenuFrame::DestroyFrom(nsIFrame* aDestructRoot)
 {
   // Kill our timer if one is active. This is not strictly necessary as
   // the pointer to this frame will be cleared from the mediator, but
@@ -389,6 +413,8 @@ nsMenuFrame::Destroy()
     mOpenTimer->Cancel();
   }
 
+  StopBlinking();
+
   // Null out the pointer to this frame in the mediator wrapper so that it 
   // doesn't try to interact with a deallocated frame.
   mTimerMediator->ClearFrame();
@@ -404,9 +430,9 @@ nsMenuFrame::Destroy()
   }
 
   if (mPopupFrame)
-    mPopupFrame->Destroy();
+    mPopupFrame->DestroyFrom(aDestructRoot);
 
-  nsBoxFrame::Destroy();
+  nsBoxFrame::DestroyFrom(aDestructRoot);
 }
 
 NS_IMETHODIMP
@@ -430,6 +456,11 @@ nsMenuFrame::HandleEvent(nsPresContext* aPresContext,
                          nsEventStatus*  aEventStatus)
 {
   NS_ENSURE_ARG_POINTER(aEventStatus);
+  if (nsEventStatus_eConsumeNoDefault == *aEventStatus ||
+      (mMenuParent && mMenuParent->IsMenuLocked())) {
+    return NS_OK;
+  }
+
   nsWeakFrame weakFrame(this);
   if (*aEventStatus == nsEventStatus_eIgnore)
     *aEventStatus = nsEventStatus_eConsumeDoDefault;
@@ -670,20 +701,21 @@ nsMenuFrame::AttributeChanged(PRInt32 aNameSpaceID,
                               nsIAtom* aAttribute,
                               PRInt32 aModType)
 {
-  nsAutoString value;
-
-  if (aAttribute == nsGkAtoms::checked) {
-    if (mType != eMenuType_Normal)
-        UpdateMenuSpecialState(PresContext());
-  } else if (aAttribute == nsGkAtoms::acceltext) {
-    // someone reset the accelText attribute, so clear the bit that says *we* set it
-    AddStateBits(NS_STATE_ACCELTEXT_IS_DERIVED);
-    BuildAcceleratorText();
-  } else if (aAttribute == nsGkAtoms::key) {
-    BuildAcceleratorText();
-  } else if (aAttribute == nsGkAtoms::type || aAttribute == nsGkAtoms::name)
-    UpdateMenuType(PresContext());
+  if (aAttribute == nsGkAtoms::acceltext && mIgnoreAccelTextChange) {
+    // Reset the flag so that only one change is ignored.
+    mIgnoreAccelTextChange = PR_FALSE;
+    return NS_OK;
+  }
 
+  if (aAttribute == nsGkAtoms::checked ||
+      aAttribute == nsGkAtoms::acceltext ||
+      aAttribute == nsGkAtoms::key ||
+      aAttribute == nsGkAtoms::type ||
+      aAttribute == nsGkAtoms::name) {
+    nsCOMPtr<nsIRunnable> event =
+      new nsMenuAttributeChangedEvent(this, aAttribute);
+    nsContentUtils::AddScriptRunner(event);
+  }
   return NS_OK;
 }
 
@@ -748,55 +780,9 @@ nsMenuFrame::DoLayout(nsBoxLayoutState& aState)
   // lay us out
   nsresult rv = nsBoxFrame::DoLayout(aState);
 
-  // layout the popup. First we need to get it.
   if (mPopupFrame) {
     PRBool sizeToPopup = IsSizedToPopup(mContent, PR_FALSE);
-    // then get its preferred size
-    nsSize prefSize = mPopupFrame->GetPrefSize(aState);
-    nsSize minSize = mPopupFrame->GetMinSize(aState); 
-    nsSize maxSize = mPopupFrame->GetMaxSize(aState);
-
-    prefSize = BoundsCheck(minSize, prefSize, maxSize);
-
-    if (sizeToPopup)
-        prefSize.width = mRect.width;
-
-    // if the pref size changed then set bounds to be the pref size
-    PRBool sizeChanged = (mPopupFrame->PreferredSize() != prefSize);
-    if (sizeChanged) {
-      mPopupFrame->SetPreferredBounds(aState, nsRect(0,0,prefSize.width, prefSize.height));
-    }
-
-    // if the menu has just been opened, or its size changed, position
-    // the popup. The flag that the popup checks in the HasOpenChanged
-    // method will get cleared in AdjustView which is called below.
-    if (IsOpen() && (sizeChanged || mPopupFrame->HasOpenChanged()))
-      mPopupFrame->SetPopupPosition(this);
-
-    // is the new size too small? Make sure we handle scrollbars correctly
-    nsIBox* child = mPopupFrame->GetChildBox();
-
-    nsRect bounds(mPopupFrame->GetRect());
-
-    nsCOMPtr<nsIScrollableFrame> scrollframe(do_QueryInterface(child));
-    if (scrollframe &&
-        scrollframe->GetScrollbarStyles().mVertical == NS_STYLE_OVERFLOW_AUTO) {
-      if (bounds.height < prefSize.height) {
-        // layout the child
-        mPopupFrame->Layout(aState);
-
-        nsMargin scrollbars = scrollframe->GetActualScrollbarSizes();
-        if (bounds.width < prefSize.width + scrollbars.left + scrollbars.right)
-        {
-          bounds.width += scrollbars.left + scrollbars.right;
-          mPopupFrame->SetBounds(aState, bounds);
-        }
-      }
-    }
-
-    // layout the child
-    mPopupFrame->Layout(aState);
-    mPopupFrame->AdjustView();
+    mPopupFrame->LayoutPopup(aState, this, sizeToPopup);
   }
 
   return rv;
@@ -906,6 +892,31 @@ nsMenuFrame::Notify(nsITimer* aTimer)
         }
       }
     }
+  } else if (aTimer == mBlinkTimer) {
+    switch (mBlinkState++) {
+      case 0:
+        NS_ASSERTION(false, "Blink timer fired while not blinking");
+        StopBlinking();
+        break;
+      case 1:
+        {
+          // Turn the highlight back on and wait for a while before closing the menu.
+          nsWeakFrame weakFrame(this);
+          mContent->SetAttr(kNameSpaceID_None, nsGkAtoms::menuactive,
+                            NS_LITERAL_STRING("true"), PR_TRUE);
+          if (weakFrame.IsAlive()) {
+            aTimer->InitWithCallback(mTimerMediator, kBlinkDelay, nsITimer::TYPE_ONE_SHOT);
+          }
+        }
+        break;
+      default:
+        if (mMenuParent) {
+          mMenuParent->LockMenuUntilClosed(PR_FALSE);
+        }
+        PassMenuCommandEventToPopupManager();
+        StopBlinking();
+        break;
+    }
   }
 
   return NS_OK;
@@ -1011,7 +1022,7 @@ nsMenuFrame::UpdateMenuSpecialState(nsPresContext* aPresContext)
 }
 
 void 
-nsMenuFrame::BuildAcceleratorText()
+nsMenuFrame::BuildAcceleratorText(PRBool aNotify)
 {
   nsAutoString accelText;
 
@@ -1027,7 +1038,7 @@ nsMenuFrame::BuildAcceleratorText()
 
   // If anything below fails, just leave the accelerator text blank.
   nsWeakFrame weakFrame(this);
-  mContent->UnsetAttr(kNameSpaceID_None, nsGkAtoms::acceltext, PR_FALSE);
+  mContent->UnsetAttr(kNameSpaceID_None, nsGkAtoms::acceltext, aNotify);
   ENSURE_TRUE(weakFrame.IsAlive());
 
   // See if we have a key node and use that instead.
@@ -1037,13 +1048,12 @@ nsMenuFrame::BuildAcceleratorText()
     return;
 
   // Turn the document into a DOM document so we can use getElementById
-  nsCOMPtr<nsIDOMDocument> domDocument(do_QueryInterface(mContent->GetDocument()));
-  if (!domDocument)
+  nsIDocument *document = mContent->GetDocument();
+  if (!document)
     return;
 
-  nsCOMPtr<nsIDOMElement> keyDOMElement;
-  domDocument->GetElementById(keyValue, getter_AddRefs(keyDOMElement));
-  if (!keyDOMElement) {
+  nsIContent *keyElement = document->GetElementById(keyValue);
+  if (!keyElement) {
 #ifdef DEBUG
     nsAutoString label;
     mContent->GetAttr(kNameSpaceID_None, nsGkAtoms::label, label);
@@ -1057,10 +1067,6 @@ nsMenuFrame::BuildAcceleratorText()
     return;
   }
 
-  nsCOMPtr<nsIContent> keyElement(do_QueryInterface(keyDOMElement));
-  if (!keyElement)
-    return;
-
   // get the string to display as accelerator text
   // check the key element's attributes in this order:
   // |keytext|, |key|, |keycode|
@@ -1078,8 +1084,9 @@ nsMenuFrame::BuildAcceleratorText()
       ToUpperCase(keyCode);
 
       nsresult rv;
-      nsCOMPtr<nsIStringBundleService> bundleService(do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv));
-      if (NS_SUCCEEDED(rv) && bundleService) {
+      nsCOMPtr<nsIStringBundleService> bundleService =
+        mozilla::services::GetStringBundleService();
+      if (bundleService) {
         nsCOMPtr<nsIStringBundle> bundle;
         rv = bundleService->CreateBundle("chrome://global/locale/keys.properties",
                                          getter_AddRefs(bundle));
@@ -1156,38 +1163,133 @@ nsMenuFrame::BuildAcceleratorText()
   nsMemory::Free(str);
 
   accelText += accelString;
-  
-  mContent->SetAttr(kNameSpaceID_None, nsGkAtoms::acceltext, accelText, PR_FALSE);
+
+  mIgnoreAccelTextChange = PR_TRUE;
+  mContent->SetAttr(kNameSpaceID_None, nsGkAtoms::acceltext, accelText, aNotify);
+  ENSURE_TRUE(weakFrame.IsAlive());
+
+  mIgnoreAccelTextChange = PR_FALSE;
 }
 
 void
 nsMenuFrame::Execute(nsGUIEvent *aEvent)
 {
-  nsWeakFrame weakFrame(this);
   // flip "checked" state if we're a checkbox menu, or an un-checked radio menu
+  PRBool needToFlipChecked = PR_FALSE;
   if (mType == eMenuType_Checkbox || (mType == eMenuType_Radio && !mChecked)) {
-    if (!mContent->AttrValueIs(kNameSpaceID_None, nsGkAtoms::autocheck,
-                               nsGkAtoms::_false, eCaseMatters)) {
-      if (mChecked) {
-        mContent->UnsetAttr(kNameSpaceID_None, nsGkAtoms::checked,
-                            PR_TRUE);
-        ENSURE_TRUE(weakFrame.IsAlive());
-      }
-      else {
-        mContent->SetAttr(kNameSpaceID_None, nsGkAtoms::checked, NS_LITERAL_STRING("true"),
-                          PR_TRUE);
-        ENSURE_TRUE(weakFrame.IsAlive());
-      }
-    }
+    needToFlipChecked = !mContent->AttrValueIs(kNameSpaceID_None, nsGkAtoms::autocheck,
+                                               nsGkAtoms::_false, eCaseMatters);
   }
 
   nsCOMPtr<nsISound> sound(do_CreateInstance("@mozilla.org/sound;1"));
   if (sound)
-    sound->PlaySystemSound(NS_SYSSOUND_MENU_EXECUTE);
+    sound->PlayEventSound(nsISound::EVENT_MENU_EXECUTE);
 
+  StartBlinking(aEvent, needToFlipChecked);
+}
+
+PRBool
+nsMenuFrame::ShouldBlink()
+{
+  PRInt32 shouldBlink = 0;
+  nsCOMPtr<nsILookAndFeel> lookAndFeel(do_GetService(kLookAndFeelCID));
+  if (lookAndFeel) {
+    lookAndFeel->GetMetric(nsILookAndFeel::eMetric_ChosenMenuItemsShouldBlink, shouldBlink);
+  }
+  if (!shouldBlink)
+    return PR_FALSE;
+
+  // Don't blink in editable menulists.
+  if (mMenuParent && mMenuParent->IsMenu()) {
+    nsMenuPopupFrame* popupFrame = static_cast<nsMenuPopupFrame*>(mMenuParent);
+    nsIFrame* parentMenu = popupFrame->GetParent();
+    if (parentMenu) {
+      nsCOMPtr<nsIDOMXULMenuListElement> menulist = do_QueryInterface(parentMenu->GetContent());
+      if (menulist) {
+        PRBool isEditable = PR_FALSE;
+        menulist->GetEditable(&isEditable);
+        return !isEditable;
+      }
+    }
+  }
+  return PR_TRUE;
+}
+
+void
+nsMenuFrame::StartBlinking(nsGUIEvent *aEvent, PRBool aFlipChecked)
+{
+  StopBlinking();
+  CreateMenuCommandEvent(aEvent, aFlipChecked);
+
+  if (!ShouldBlink()) {
+    PassMenuCommandEventToPopupManager();
+    return;
+  }
+
+  // Blink off.
+  nsWeakFrame weakFrame(this);
+  mContent->UnsetAttr(kNameSpaceID_None, nsGkAtoms::menuactive, PR_TRUE);
+  if (!weakFrame.IsAlive())
+    return;
+
+  if (mMenuParent) {
+    // Make this menu ignore events from now on.
+    mMenuParent->LockMenuUntilClosed(PR_TRUE);
+  }
+
+  // Set up a timer to blink back on.
+  mBlinkTimer = do_CreateInstance("@mozilla.org/timer;1");
+  mBlinkTimer->InitWithCallback(mTimerMediator, kBlinkDelay, nsITimer::TYPE_ONE_SHOT);
+  mBlinkState = 1;
+}
+
+void
+nsMenuFrame::StopBlinking()
+{
+  mBlinkState = 0;
+  if (mBlinkTimer) {
+    mBlinkTimer->Cancel();
+    mBlinkTimer = nsnull;
+  }
+  mDelayedMenuCommandEvent = nsnull;
+}
+
+void
+nsMenuFrame::CreateMenuCommandEvent(nsGUIEvent *aEvent, PRBool aFlipChecked)
+{
+  // Create a trusted event if the triggering event was trusted, or if
+  // we're called from chrome code (since at least one of our caller
+  // passes in a null event).
+  PRBool isTrusted = aEvent ? NS_IS_TRUSTED_EVENT(aEvent) :
+                              nsContentUtils::IsCallerChrome();
+
+  PRBool shift = PR_FALSE, control = PR_FALSE, alt = PR_FALSE, meta = PR_FALSE;
+  if (aEvent && (aEvent->eventStructType == NS_MOUSE_EVENT ||
+                 aEvent->eventStructType == NS_KEY_EVENT)) {
+    shift = static_cast<nsInputEvent *>(aEvent)->isShift;
+    control = static_cast<nsInputEvent *>(aEvent)->isControl;
+    alt = static_cast<nsInputEvent *>(aEvent)->isAlt;
+    meta = static_cast<nsInputEvent *>(aEvent)->isMeta;
+  }
+
+  // Because the command event is firing asynchronously, a flag is needed to
+  // indicate whether user input is being handled. This ensures that a popup
+  // window won't get blocked.
+  PRBool userinput = nsEventStateManager::IsHandlingUserInput();
+
+  mDelayedMenuCommandEvent =
+    new nsXULMenuCommandEvent(mContent, isTrusted, shift, control, alt, meta,
+                              userinput, aFlipChecked);
+}
+
+void
+nsMenuFrame::PassMenuCommandEventToPopupManager()
+{
   nsXULPopupManager* pm = nsXULPopupManager::GetInstance();
-  if (pm && mMenuParent)
-    pm->ExecuteMenu(mContent, aEvent);
+  if (pm && mMenuParent && mDelayedMenuCommandEvent) {
+    pm->ExecuteMenu(mContent, mDelayedMenuCommandEvent);
+  }
+  mDelayedMenuCommandEvent = nsnull;
 }
 
 NS_IMETHODIMP
@@ -1214,12 +1316,11 @@ nsMenuFrame::RemoveFrame(nsIAtom*        aListName,
 NS_IMETHODIMP
 nsMenuFrame::InsertFrames(nsIAtom*        aListName,
                           nsIFrame*       aPrevFrame,
-                          nsIFrame*       aFrameList)
+                          nsFrameList&    aFrameList)
 {
   if (!mPopupFrame && (!aListName || aListName == nsGkAtoms::popupList)) {
-    aFrameList = SetPopupFrame(aFrameList);
+    SetPopupFrame(aFrameList);
     if (mPopupFrame) {
-
 #ifdef DEBUG_LAYOUT
       nsBoxLayoutState state(PresContext());
       SetDebug(state, aFrameList, mState & NS_STATE_CURRENTLY_IN_DEBUG);
@@ -1228,12 +1329,10 @@ nsMenuFrame::InsertFrames(nsIAtom*        aListName,
       PresContext()->PresShell()->
         FrameNeedsReflow(this, nsIPresShell::eTreeChange,
                          NS_FRAME_HAS_DIRTY_CHILDREN);
-
-      return NS_OK;
     }
   }
 
-  if (!aFrameList)
+  if (aFrameList.IsEmpty())
     return NS_OK;
 
   if (NS_UNLIKELY(aPrevFrame == mPopupFrame)) {
@@ -1245,13 +1344,10 @@ nsMenuFrame::InsertFrames(nsIAtom*        aListName,
 
 NS_IMETHODIMP
 nsMenuFrame::AppendFrames(nsIAtom*        aListName,
-                          nsIFrame*       aFrameList)
+                          nsFrameList&    aFrameList)
 {
-  if (!aFrameList)
-    return NS_OK;
-
   if (!mPopupFrame && (!aListName || aListName == nsGkAtoms::popupList)) {
-    aFrameList = SetPopupFrame(aFrameList);
+    SetPopupFrame(aFrameList);
     if (mPopupFrame) {
 
 #ifdef DEBUG_LAYOUT
@@ -1261,12 +1357,10 @@ nsMenuFrame::AppendFrames(nsIAtom*        aListName,
       PresContext()->PresShell()->
         FrameNeedsReflow(this, nsIPresShell::eTreeChange,
                          NS_FRAME_HAS_DIRTY_CHILDREN);
-
-      return NS_OK;
     }
   }
 
-  if (!aFrameList)
+  if (aFrameList.IsEmpty())
     return NS_OK;
 
   return nsBoxFrame::AppendFrames(aListName, aFrameList); 
@@ -1276,13 +1370,21 @@ PRBool
 nsMenuFrame::SizeToPopup(nsBoxLayoutState& aState, nsSize& aSize)
 {
   if (!IsCollapsed(aState)) {
+    PRBool widthSet, heightSet;
     nsSize tmpSize(-1, 0);
-    nsIBox::AddCSSPrefSize(aState, this, tmpSize);
-    if (tmpSize.width == -1 && GetFlex(aState) == 0) {
+    nsIBox::AddCSSPrefSize(this, tmpSize, widthSet, heightSet);
+    if (!widthSet && GetFlex(aState) == 0) {
       if (!mPopupFrame)
         return PR_FALSE;
       tmpSize = mPopupFrame->GetPrefSize(aState);
       aSize.width = tmpSize.width;
+
+      // if there is a scroll frame, add the desired width of the scrollbar as well
+      nsIScrollableFrame* scrollFrame = do_QueryFrame(mPopupFrame->GetFirstChild(nsnull));
+      if (scrollFrame) {
+        aSize.width += scrollFrame->GetDesiredScrollbarSizes(&aState).LeftRight();
+      }
+
       return PR_TRUE;
     }
   }
@@ -1343,20 +1445,19 @@ nsMenuFrame::SetActiveChild(nsIDOMElement* aChild)
 
   nsCOMPtr<nsIContent> child(do_QueryInterface(aChild));
 
-  nsIFrame* kid = PresContext()->PresShell()->GetPrimaryFrameFor(child);
+  nsIFrame* kid = child->GetPrimaryFrame();
   if (kid && kid->GetType() == nsGkAtoms::menuFrame)
     mPopupFrame->ChangeMenuItem(static_cast<nsMenuFrame *>(kid), PR_FALSE);
   return NS_OK;
 }
 
-nsIScrollableView* nsMenuFrame::GetScrollableView()
+nsIScrollableFrame* nsMenuFrame::GetScrollTargetFrame()
 {
   if (!mPopupFrame)
     return nsnull;
-
   nsIFrame* childFrame = mPopupFrame->GetFirstChild(nsnull);
   if (childFrame)
-    return mPopupFrame->GetScrollableView(childFrame);
+    return mPopupFrame->GetScrollFrame(childFrame);
   return nsnull;
 }
 
