@@ -90,7 +90,6 @@
 #include "nsIPersistentProperties2.h"
 #include "nsISyncStreamListener.h"
 #include "nsInterfaceRequestorAgg.h"
-#include "nsInt64.h"
 #include "nsINetUtil.h"
 #include "nsIURIWithPrincipal.h"
 #include "nsIAuthPrompt.h"
@@ -109,9 +108,7 @@
 #include "nsISocketProvider.h"
 #include "mozilla/Services.h"
 
-#ifdef MOZ_IPC
 #include "nsIRedirectChannelRegistrar.h"
-#endif
 
 #ifdef MOZILLA_INTERNAL_API
 
@@ -490,8 +487,8 @@ NS_NewInputStreamChannel(nsIChannel      **result,
 inline nsresult
 NS_NewInputStreamPump(nsIInputStreamPump **result,
                       nsIInputStream      *stream,
-                      PRInt64              streamPos = nsInt64(-1),
-                      PRInt64              streamLen = nsInt64(-1),
+                      PRInt64              streamPos = PRInt64(-1),
+                      PRInt64              streamLen = PRInt64(-1),
                       PRUint32             segsize = 0,
                       PRUint32             segcount = 0,
                       PRBool               closeWhenDone = PR_FALSE)
@@ -1040,7 +1037,7 @@ NS_BackgroundInputStream(nsIInputStream **result,
         do_GetService(NS_STREAMTRANSPORTSERVICE_CONTRACTID, &rv);
     if (NS_SUCCEEDED(rv)) {
         nsCOMPtr<nsITransport> inTransport;
-        rv = sts->CreateInputTransport(stream, nsInt64(-1), nsInt64(-1),
+        rv = sts->CreateInputTransport(stream, PRInt64(-1), PRInt64(-1),
                                        PR_TRUE, getter_AddRefs(inTransport));
         if (NS_SUCCEEDED(rv))
             rv = inTransport->OpenInputStream(nsITransport::OPEN_BLOCKING,
@@ -1064,7 +1061,7 @@ NS_BackgroundOutputStream(nsIOutputStream **result,
         do_GetService(NS_STREAMTRANSPORTSERVICE_CONTRACTID, &rv);
     if (NS_SUCCEEDED(rv)) {
         nsCOMPtr<nsITransport> inTransport;
-        rv = sts->CreateOutputTransport(stream, nsInt64(-1), nsInt64(-1),
+        rv = sts->CreateOutputTransport(stream, PRInt64(-1), PRInt64(-1),
                                         PR_TRUE, getter_AddRefs(inTransport));
         if (NS_SUCCEEDED(rv))
             rv = inTransport->OpenOutputStream(nsITransport::OPEN_BLOCKING,
@@ -1784,7 +1781,6 @@ NS_IsInternalSameURIRedirect(nsIChannel *aOldChannel,
   return NS_SUCCEEDED(oldURI->Equals(newURI, &res)) && res;
 }
 
-#ifdef MOZ_IPC
 inline nsresult
 NS_LinkRedirectChannels(PRUint32 channelId,
                         nsIParentChannel *parentChannel,
@@ -1800,7 +1796,6 @@ NS_LinkRedirectChannels(PRUint32 channelId,
                                  parentChannel,
                                  _result);
 }
-#endif // MOZ_IPC
 
 /**
  * Helper function to create a random URL string that's properly formed
