@@ -47,6 +47,7 @@
 #include "nsBaseWidget.h"
 #include "nsPIWidgetCocoa.h"
 #include "nsAutoPtr.h"
+#include "nsCocoaUtils.h"
 
 class nsCocoaWindow;
 class nsChildView;
@@ -58,6 +59,42 @@ typedef struct _nsCocoaWindowList {
   nsCocoaWindow *window; // Weak
 } nsCocoaWindowList;
 
+// NSWindow subclass that is the base class for all of our own window classes.
+// Among other things, this class handles the storage of those settings that
+// need to be persisted across window destruction and reconstruction, i.e. when
+// switching to and from fullscreen mode.
+// We don't save shadow, transparency mode or background color because it's not
+// worth the hassle - Gecko will reset them anyway as soon as the window is
+// resized.
+@interface BaseWindow : NSWindow
+{
+  // Data Storage
+  NSMutableDictionary* mState;
+  BOOL mDrawsIntoWindowFrame;
+  NSColor* mActiveTitlebarColor;
+  NSColor* mInactiveTitlebarColor;
+
+  // Shadow
+  BOOL mScheduledShadowInvalidation;
+
+  // DPI cache. Getting the physical screen size (CGDisplayScreenSize)
+  // is ridiculously slow, so we cache it in the toplevel window for all
+  // descendants to use.
+  float mDPI;
+}
+
+- (void)importState:(NSDictionary*)aState;
+- (NSMutableDictionary*)exportState;
+- (void)setDrawsContentsIntoWindowFrame:(BOOL)aState;
+- (BOOL)drawsContentsIntoWindowFrame;
+- (void)setTitlebarColor:(NSColor*)aColor forActiveWindow:(BOOL)aActive;
+- (NSColor*)titlebarColorForActiveWindow:(BOOL)aActive;
+
+- (void)deferredInvalidateShadow;
+- (void)invalidateShadow;
+- (float)getDPI;
+
+@end
 
 @interface NSWindow (Undocumented)
 
@@ -66,7 +103,7 @@ typedef struct _nsCocoaWindowList {
 // (and put it back in the "window cache").  One way to do this, which Apple
 // often uses, is to set the "window number" to '-1' and then back to its
 // original value.
-- (void)_setWindowNumber:(int)aNumber;
+- (void)_setWindowNumber:(NSInteger)aNumber;
 
 // If we set the window's stylemask to be textured, the corners on the bottom of
 // the window are rounded by default. We use this private method to make
@@ -75,22 +112,21 @@ typedef struct _nsCocoaWindowList {
 
 @end
 
-
-@interface PopupWindow : NSWindow
+@interface PopupWindow : BaseWindow
 {
 @private
   BOOL mIsContextMenu;
 }
 
-- (id)initWithContentRect:(NSRect)contentRect styleMask:(unsigned int)styleMask
+- (id)initWithContentRect:(NSRect)contentRect styleMask:(NSUInteger)styleMask
       backing:(NSBackingStoreType)bufferingType defer:(BOOL)deferCreation;
 - (BOOL)isContextMenu;
 - (void)setIsContextMenu:(BOOL)flag;
+- (BOOL)canBecomeMainWindow;
 
 @end
 
-
-@interface BorderlessWindow : NSWindow
+@interface BorderlessWindow : BaseWindow
 {
 }
 
@@ -99,14 +135,18 @@ typedef struct _nsCocoaWindowList {
 
 @end
 
-
+#if defined( MAC_OS_X_VERSION_10_6 ) && ( MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6 )
+@interface WindowDelegate : NSObject <NSWindowDelegate>
+#else
 @interface WindowDelegate : NSObject
+#endif
 {
   nsCocoaWindow* mGeckoWindow; // [WEAK] (we are owned by the window)
-  // Used to avoid duplication when we send NS_ACTIVATE/NS_GOTFOCUS and
-  // NS_DEACTIVATE/NS_LOSTFOCUS to Gecko for toplevel widgets.  Starts out
+  // Used to avoid duplication when we send NS_ACTIVATE and
+  // NS_DEACTIVATE to Gecko for toplevel widgets.  Starts out
   // PR_FALSE.
   PRBool mToplevelActiveState;
+  BOOL mHasEverBeenZoomed;
 }
 + (void)paintMenubarForWindow:(NSWindow*)aWindow;
 - (id)initWithGeckoWindow:(nsCocoaWindow*)geckoWind;
@@ -125,45 +165,35 @@ struct UnifiedGradientInfo {
   BOOL drawTitlebar; // NO for toolbar, YES for titlebar
 };
 
+@class ToolbarWindow;
+
 // NSColor subclass that allows us to draw separate colors both in the titlebar 
 // and for background of the window.
 @interface TitlebarAndBackgroundColor : NSColor
 {
-  NSColor *mActiveTitlebarColor;
-  NSColor *mInactiveTitlebarColor;
-  NSColor *mBackgroundColor;
-  NSWindow *mWindow; // [WEAK] (we are owned by the window)
+  ToolbarWindow *mWindow; // [WEAK] (we are owned by the window)
 }
 
-- (id)initWithActiveTitlebarColor:(NSColor*)aActiveTitlebarColor
-            inactiveTitlebarColor:(NSColor*)aInactiveTitlebarColor
-                  backgroundColor:(NSColor*)aBackgroundColor
-                        forWindow:(NSWindow*)aWindow;
+- (id)initWithWindow:(ToolbarWindow*)aWindow;
 
-// Pass nil here to get the default appearance.
-- (void)setTitlebarColor:(NSColor*)aColor forActiveWindow:(BOOL)aActive;
-- (NSColor*)activeTitlebarColor;
-- (NSColor*)inactiveTitlebarColor;
-
-- (void)setBackgroundColor:(NSColor*)aColor;
-- (NSColor*)backgroundColor;
-
-- (NSWindow*)window;
 @end
 
 // NSWindow subclass for handling windows with toolbars.
-@interface ToolbarWindow : NSWindow
+@interface ToolbarWindow : BaseWindow
 {
   TitlebarAndBackgroundColor *mColor;
   float mUnifiedToolbarHeight;
+  NSColor *mBackgroundColor;
 }
+// Pass nil here to get the default appearance.
 - (void)setTitlebarColor:(NSColor*)aColor forActiveWindow:(BOOL)aActive;
-- (void)setUnifiedToolbarHeight:(float)aToolbarHeight;
+- (void)setUnifiedToolbarHeight:(float)aHeight;
 - (float)unifiedToolbarHeight;
 - (float)titlebarHeight;
-// This method is also available on NSWindows (via a category), and is the 
-// preferred way to check the background color of a window.
-- (NSColor*)windowBackgroundColor;
+- (NSRect)titlebarRect;
+- (void)setTitlebarNeedsDisplayInRect:(NSRect)aRect sync:(BOOL)aSync;
+- (void)setTitlebarNeedsDisplayInRect:(NSRect)aRect;
+- (void)setDrawsContentsIntoWindowFrame:(BOOL)aState;
 @end
 
 class nsCocoaWindow : public nsBaseWidget, public nsPIWidgetCocoa
@@ -180,16 +210,9 @@ public:
     NS_DECL_ISUPPORTS_INHERITED
     NS_DECL_NSPIWIDGETCOCOA
       
-    NS_IMETHOD              Create(nsNativeWidget aParent,
-                                   const nsRect &aRect,
-                                   EVENT_CALLBACK aHandleEventFunction,
-                                   nsIDeviceContext *aContext,
-                                   nsIAppShell *aAppShell = nsnull,
-                                   nsIToolkit *aToolkit = nsnull,
-                                   nsWidgetInitData *aInitData = nsnull);
-
     NS_IMETHOD              Create(nsIWidget* aParent,
-                                   const nsRect &aRect,
+                                   nsNativeWidget aNativeParent,
+                                   const nsIntRect &aRect,
                                    EVENT_CALLBACK aHandleEventFunction,
                                    nsIDeviceContext *aContext,
                                    nsIAppShell *aAppShell = nsnull,
@@ -197,32 +220,18 @@ public:
                                    nsWidgetInitData *aInitData = nsnull);
 
     NS_IMETHOD              Destroy();
-     // Utility method for implementing both Create(nsIWidget ...) and
-     // Create(nsNativeWidget...)
-
-    virtual nsresult        StandardCreate(nsIWidget *aParent,
-                                    const nsRect &aRect,
-                                    EVENT_CALLBACK aHandleEventFunction,
-                                    nsIDeviceContext *aContext,
-                                    nsIAppShell *aAppShell,
-                                    nsIToolkit *aToolkit,
-                                    nsWidgetInitData *aInitData,
-                                    nsNativeWidget aNativeWindow = nsnull);
 
     NS_IMETHOD              Show(PRBool aState);
     virtual nsIWidget*      GetSheetWindowParent(void);
-    NS_IMETHOD              AddEventListener(nsIEventListener * aListener);
     NS_IMETHOD              Enable(PRBool aState);
     NS_IMETHOD              IsEnabled(PRBool *aState);
     NS_IMETHOD              SetModal(PRBool aState);
     NS_IMETHOD              IsVisible(PRBool & aState);
     NS_IMETHOD              SetFocus(PRBool aState=PR_FALSE);
-    NS_IMETHOD              SetMenuBar(void* aMenuBar);
-    virtual nsMenuBarX*     GetMenuBar();
-    NS_IMETHOD              ShowMenuBar(PRBool aShow);
-    NS_IMETHOD WidgetToScreen(const nsRect& aOldRect, nsRect& aNewRect);
-    NS_IMETHOD ScreenToWidget(const nsRect& aOldRect, nsRect& aNewRect);
-    
+    virtual nsIntPoint WidgetToScreenOffset();
+    virtual nsIntPoint GetClientOffset();
+    virtual nsIntSize ClientToWindowSize(const nsIntSize& aClientSize);
+
     virtual void* GetNativeData(PRUint32 aDataType) ;
 
     NS_IMETHOD              ConstrainPosition(PRBool aAllowSlop,
@@ -231,76 +240,107 @@ public:
     NS_IMETHOD              PlaceBehind(nsTopLevelWidgetZPlacement aPlacement,
                                         nsIWidget *aWidget, PRBool aActivate);
     NS_IMETHOD              SetSizeMode(PRInt32 aMode);
-
+    NS_IMETHOD              HideWindowChrome(PRBool aShouldHide);
+    NS_IMETHOD              MakeFullScreen(PRBool aFullScreen);
     NS_IMETHOD              Resize(PRInt32 aWidth,PRInt32 aHeight, PRBool aRepaint);
     NS_IMETHOD              Resize(PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight, PRBool aRepaint);
-    NS_IMETHOD              GetScreenBounds(nsRect &aRect);
-    virtual PRBool          OnPaint(nsPaintEvent &event);
-    void                    ReportSizeEvent(NSRect *overrideRect = nsnull);
+    NS_IMETHOD              GetClientBounds(nsIntRect &aRect);
+    NS_IMETHOD              GetScreenBounds(nsIntRect &aRect);
+    void                    ReportMoveEvent();
+    void                    ReportSizeEvent();
+    NS_IMETHOD              SetCursor(nsCursor aCursor);
+    NS_IMETHOD              SetCursor(imgIContainer* aCursor, PRUint32 aHotspotX, PRUint32 aHotspotY);
 
     NS_IMETHOD              SetTitle(const nsAString& aTitle);
 
-    NS_IMETHOD Invalidate(const nsRect & aRect, PRBool aIsSynchronous);
-    NS_IMETHOD Invalidate(PRBool aIsSynchronous);
+    NS_IMETHOD Invalidate(const nsIntRect &aRect, PRBool aIsSynchronous);
     NS_IMETHOD Update();
-    NS_IMETHOD Scroll(PRInt32 aDx, PRInt32 aDy, nsRect *alCipRect) { return NS_OK; }
-    NS_IMETHOD SetColorMap(nsColorMap *aColorMap) { return NS_OK; }
-    NS_IMETHOD BeginResizingChildren(void) { return NS_OK; }
-    NS_IMETHOD EndResizingChildren(void) { return NS_OK; }
-    NS_IMETHOD GetPreferredSize(PRInt32& aWidth, PRInt32& aHeight) { return NS_OK; }
-    NS_IMETHOD SetPreferredSize(PRInt32 aWidth, PRInt32 aHeight) { return NS_OK; }
+    virtual nsresult ConfigureChildren(const nsTArray<Configuration>& aConfigurations);
+    virtual LayerManager* GetLayerManager(bool *aAllowRetaining = nsnull);
+    virtual LayerManager* GetLayerManager(LayerManagerPersistence aPersistence = LAYER_MANAGER_CURRENT,
+                                          bool* aAllowRetaining = nsnull);
     NS_IMETHOD DispatchEvent(nsGUIEvent* event, nsEventStatus & aStatus) ;
-    NS_IMETHOD CaptureRollupEvents(nsIRollupListener * aListener, PRBool aDoCapture, PRBool aConsumeRollupEvent);
+    NS_IMETHOD CaptureRollupEvents(nsIRollupListener * aListener, nsIMenuRollup * aMenuRollup,
+                                   PRBool aDoCapture, PRBool aConsumeRollupEvent);
     NS_IMETHOD GetAttention(PRInt32 aCycleCount);
     virtual PRBool HasPendingInputEvent();
     virtual nsTransparencyMode GetTransparencyMode();
     virtual void SetTransparencyMode(nsTransparencyMode aMode);
     NS_IMETHOD SetWindowShadowStyle(PRInt32 aStyle);
+    virtual void SetShowsToolbarButton(PRBool aShow);
     NS_IMETHOD SetWindowTitlebarColor(nscolor aColor, PRBool aActive);
+    virtual void SetDrawsInTitlebar(PRBool aState);
+    virtual nsresult SynthesizeNativeMouseEvent(nsIntPoint aPoint,
+                                                PRUint32 aNativeMessage,
+                                                PRUint32 aModifierFlags);
 
-    // dispatch an NS_SIZEMODE event on miniaturize or deminiaturize
-    void DispatchSizeModeEvent(nsSizeMode aSizeMode);
+    void DispatchSizeModeEvent();
 
     virtual gfxASurface* GetThebesSurface();
 
     // be notified that a some form of drag event needs to go into Gecko
     virtual PRBool DragEvent(unsigned int aMessage, Point aMouseGlobal, UInt16 aKeyModifiers);
 
-    // Helpers to prevent recursive resizing during live-resize
-    PRBool IsResizing () const { return mIsResizing; }
-    void StartResizing () { mIsResizing = PR_TRUE; }
-    void StopResizing () { mIsResizing = PR_FALSE; }
-
     PRBool HasModalDescendents() { return mNumModalDescendents > 0; }
     NSWindow *GetCocoaWindow() { return mWindow; }
 
+    void SetMenuBar(nsMenuBarX* aMenuBar);
+    nsMenuBarX *GetMenuBar();
+
     // nsIKBStateControl interface
     NS_IMETHOD ResetInputState();
     
-    void MakeBackgroundTransparent(PRBool aTransparent);
-
     NS_IMETHOD BeginSecureKeyboardInput();
     NS_IMETHOD EndSecureKeyboardInput();
 
-    static void UnifiedShading(void* aInfo, const float* aIn, float* aOut);
+    static void UnifiedShading(void* aInfo, const CGFloat* aIn, CGFloat* aOut);
 
+    void SetPopupWindowLevel();
+
+    PRBool IsChildInFailingLeftClickThrough(NSView *aChild);
+    PRBool ShouldFocusPlugin();
+
+    NS_IMETHOD         ReparentNativeWidget(nsIWidget* aNewParent);
 protected:
-  
+
+  nsresult             CreateNativeWindow(const NSRect &aRect,
+                                          nsBorderStyle aBorderStyle,
+                                          PRBool aRectIsFrameRect);
+  nsresult             CreatePopupContentView(const nsIntRect &aRect,
+                                              EVENT_CALLBACK aHandleEventFunction,
+                                              nsIDeviceContext *aContext,
+                                              nsIAppShell *aAppShell,
+                                              nsIToolkit *aToolkit);
+  void                 DestroyNativeWindow();
+  void                 AdjustWindowShadow();
+  void                 SetUpWindowFilter();
+  void                 CleanUpWindowFilter();
+  void                 UpdateBounds();
+
+  virtual already_AddRefed<nsIWidget>
+  AllocateChildPopupWidget()
+  {
+    static NS_DEFINE_IID(kCPopUpCID, NS_POPUP_CID);
+    nsCOMPtr<nsIWidget> widget = do_CreateInstance(kCPopUpCID);
+    return widget.forget();
+  }
+
   nsIWidget*           mParent;         // if we're a popup, this is our parent [WEAK]
-  NSWindow*            mWindow;         // our cocoa window [STRONG]
+  BaseWindow*          mWindow;         // our cocoa window [STRONG]
   WindowDelegate*      mDelegate;       // our delegate for processing window msgs [STRONG]
   nsRefPtr<nsMenuBarX> mMenuBar;
   NSWindow*            mSheetWindowParent; // if this is a sheet, this is the NSWindow it's attached to
   nsChildView*         mPopupContentView; // if this is a popup, this is its content widget
+  PRInt32              mShadowStyle;
+  NSUInteger           mWindowFilter;
 
-  PRPackedBool         mIsResizing;     // we originated the resize, prevent infinite recursion
   PRPackedBool         mWindowMadeHere; // true if we created the window, false for embedding
   PRPackedBool         mSheetNeedsShow; // if this is a sheet, are we waiting to be shown?
                                         // this is used for sibling sheet contention only
+  PRPackedBool         mFullScreen;
   PRPackedBool         mModal;
 
   PRInt32              mNumModalDescendents;
 };
 
-
 #endif // nsCocoaWindow_h_
