@@ -750,7 +750,7 @@ nsBaseWidget::AutoLayerManagerSetup::AutoLayerManagerSetup(
   : mWidget(aWidget)
 {
   BasicLayerManager* manager =
-    static_cast<BasicLayerManager*>(mWidget->GetLayerManager(nsnull));
+    static_cast<BasicLayerManager*>(mWidget->GetLayerManager());
   if (manager) {
     NS_ASSERTION(manager->GetBackendType() == LayerManager::LAYERS_BASIC,
       "AutoLayerManagerSetup instantiated for non-basic layer backend!");
@@ -761,7 +761,7 @@ nsBaseWidget::AutoLayerManagerSetup::AutoLayerManagerSetup(
 nsBaseWidget::AutoLayerManagerSetup::~AutoLayerManagerSetup()
 {
   BasicLayerManager* manager =
-    static_cast<BasicLayerManager*>(mWidget->GetLayerManager(nsnull));
+    static_cast<BasicLayerManager*>(mWidget->GetLayerManager());
   if (manager) {
     NS_ASSERTION(manager->GetBackendType() == LayerManager::LAYERS_BASIC,
       "AutoLayerManagerSetup instantiated for non-basic layer backend!");
@@ -863,11 +863,6 @@ nsBaseWidget::GetShouldAccelerate()
   return mUseAcceleratedRendering;
 }
 
-LayerManager* nsBaseWidget::GetLayerManager(bool* aAllowRetaining)
-{
-  return GetLayerManager(LAYER_MANAGER_CURRENT, aAllowRetaining);
-}
-
 LayerManager* nsBaseWidget::GetLayerManager(LayerManagerPersistence,
                                             bool* aAllowRetaining)
 {
@@ -906,11 +901,7 @@ LayerManager* nsBaseWidget::GetLayerManager(LayerManagerPersistence,
 
 BasicLayerManager* nsBaseWidget::CreateBasicLayerManager()
 {
-#if !defined(MOZ_IPC)
-      return new BasicLayerManager(this);
-#else
       return new BasicShadowLayerManager(this);
-#endif
 }
 
 //-------------------------------------------------------------------------
